/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


import java.io.IOException;

import jgromacs.data.Atom;
import jgromacs.data.Point3D;
import jgromacs.data.Structure;
import jgromacs.db.AtomType;
import jgromacs.io.IOData;
import junit.framework.TestCase;

import org.junit.Test;

public class AtomTest extends TestCase{

	@Test
	public void testIsHydrogenAtom() {
			
			Atom atom = new Atom();
			
			atom.setAtomType(new AtomType("H"));
			assertTrue(atom.isHydrogenAtom());
			
			atom.setAtomType(new AtomType("Ne"));
			assertTrue(!atom.isHydrogenAtom());
			
			atom.setAtomType(new AtomType("F"));
			assertTrue(!atom.isHydrogenAtom());
			
			atom.setAtomType(new AtomType("Hg"));
			assertTrue(!atom.isHydrogenAtom());
			
	}

	@Test
	public void testIsHeavyAtom() {
		
		Atom atom = new Atom();
		
		atom.setAtomType(new AtomType("H"));
		assertTrue(!atom.isHeavyAtom());
		
		atom.setAtomType(new AtomType("C"));
		assertTrue(atom.isHeavyAtom());
		
		atom.setAtomType(new AtomType("N"));
		assertTrue(atom.isHeavyAtom());
		
		atom.setAtomType(new AtomType("O"));
		assertTrue(atom.isHeavyAtom());
		
		atom.setAtomType(new AtomType("S"));
		assertTrue(atom.isHeavyAtom());
	
	}

	@Test
	public void testIsAlphaCarbon() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(14).isAlphaCarbon());
			assertTrue(s.getAtomByIndex(53).isAlphaCarbon());
			assertTrue(s.getAtomByIndex(204).isAlphaCarbon());
			assertTrue(s.getAtomByIndex(1186).isAlphaCarbon());
			
			assertTrue(!s.getAtomByIndex(5).isAlphaCarbon());	
			assertTrue(!s.getAtomByIndex(95).isAlphaCarbon());	
			assertTrue(!s.getAtomByIndex(325).isAlphaCarbon());
			assertTrue(!s.getAtomByIndex(6).isAlphaCarbon());
			
			} catch (IOException e) {
				fail(e.toString());
			}
		
	}

	@Test
	public void testIsCTerminalCarbon() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(147).isCTerminalCarbon());
			assertTrue(s.getAtomByIndex(358).isCTerminalCarbon());
			assertTrue(s.getAtomByIndex(1387).isCTerminalCarbon());
			assertTrue(s.getAtomByIndex(1426).isCTerminalCarbon());
			
			assertTrue(!s.getAtomByIndex(69).isCTerminalCarbon());	
			assertTrue(!s.getAtomByIndex(468).isCTerminalCarbon());	
			assertTrue(!s.getAtomByIndex(793).isCTerminalCarbon());
			assertTrue(!s.getAtomByIndex(1432).isCTerminalCarbon());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testIsBetaCarbon() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(70).isBetaCarbon());
			assertTrue(s.getAtomByIndex(437).isBetaCarbon());
			assertTrue(s.getAtomByIndex(757).isBetaCarbon());
			assertTrue(s.getAtomByIndex(1261).isBetaCarbon());
			
			assertTrue(!s.getAtomByIndex(107).isBetaCarbon());	
			assertTrue(!s.getAtomByIndex(215).isBetaCarbon());	
			assertTrue(!s.getAtomByIndex(602).isBetaCarbon());
			assertTrue(!s.getAtomByIndex(956).isBetaCarbon());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}


	@Test
	public void testIsGammaCarbon() {
	
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(19).isGammaCarbon());
			assertTrue(s.getAtomByIndex(72).isGammaCarbon());
			assertTrue(s.getAtomByIndex(75).isGammaCarbon());
			assertTrue(s.getAtomByIndex(560).isGammaCarbon());
			
			assertTrue(!s.getAtomByIndex(578).isGammaCarbon());	
			assertTrue(!s.getAtomByIndex(895).isGammaCarbon());	
			assertTrue(!s.getAtomByIndex(1109).isGammaCarbon());
			assertTrue(!s.getAtomByIndex(1163).isGammaCarbon());
			
			} catch (IOException e) {
				fail(e.toString());
			}
		
	}
	
	@Test
	public void testIsDeltaCarbon() {
	
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(22).isDeltaCarbon());
			assertTrue(s.getAtomByIndex(95).isDeltaCarbon());
			assertTrue(s.getAtomByIndex(252).isDeltaCarbon());
			assertTrue(s.getAtomByIndex(1329).isDeltaCarbon());
			
			assertTrue(!s.getAtomByIndex(269).isDeltaCarbon());	
			assertTrue(!s.getAtomByIndex(276).isDeltaCarbon());	
			assertTrue(!s.getAtomByIndex(555).isDeltaCarbon());
			assertTrue(!s.getAtomByIndex(886).isDeltaCarbon());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}
	
	@Test
	public void testIsEpsilonCarbon() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(193).isEpsilonCarbon());
			assertTrue(s.getAtomByIndex(281).isEpsilonCarbon());
			assertTrue(s.getAtomByIndex(447).isEpsilonCarbon());
			assertTrue(s.getAtomByIndex(1234).isEpsilonCarbon());
			
			assertTrue(!s.getAtomByIndex(627).isEpsilonCarbon());	
			assertTrue(!s.getAtomByIndex(865).isEpsilonCarbon());	
			assertTrue(!s.getAtomByIndex(1014).isEpsilonCarbon());
			assertTrue(!s.getAtomByIndex(1438).isEpsilonCarbon());
			
			} catch (IOException e) {
				fail(e.toString());
			}
		
	}
	
	@Test
	public void testIsZetaCarbon() {
		
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(100).isZetaCarbon());
			assertTrue(s.getAtomByIndex(449).isZetaCarbon());
			assertTrue(s.getAtomByIndex(997).isZetaCarbon());
			assertTrue(s.getAtomByIndex(1404).isZetaCarbon());
			
			assertTrue(!s.getAtomByIndex(281).isZetaCarbon());	
			assertTrue(!s.getAtomByIndex(560).isZetaCarbon());	
			assertTrue(!s.getAtomByIndex(1261).isZetaCarbon());
			assertTrue(!s.getAtomByIndex(1329).isZetaCarbon());
			
			} catch (IOException e) {
				fail(e.toString());
			}
		
	}
	


	@Test
	public void testIsCarbonylOxygen() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(452).isCarbonylOxygen());
			assertTrue(s.getAtomByIndex(719).isCarbonylOxygen());
			assertTrue(s.getAtomByIndex(1019).isCarbonylOxygen());
			assertTrue(s.getAtomByIndex(1084).isCarbonylOxygen());
			
			assertTrue(!s.getAtomByIndex(176).isCarbonylOxygen());	
			assertTrue(!s.getAtomByIndex(413).isCarbonylOxygen());	
			assertTrue(!s.getAtomByIndex(628).isCarbonylOxygen());
			assertTrue(!s.getAtomByIndex(1103).isCarbonylOxygen());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testIsNTerminalNitrogen() {
	
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(300).isNTerminalNitrogen());
			assertTrue(s.getAtomByIndex(360).isNTerminalNitrogen());
			assertTrue(s.getAtomByIndex(696).isNTerminalNitrogen());
			assertTrue(s.getAtomByIndex(1053).isNTerminalNitrogen());
			
			assertTrue(!s.getAtomByIndex(238).isNTerminalNitrogen());	
			assertTrue(!s.getAtomByIndex(393).isNTerminalNitrogen());	
			assertTrue(!s.getAtomByIndex(913).isNTerminalNitrogen());
			assertTrue(!s.getAtomByIndex(1424).isNTerminalNitrogen());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testIsBackboneAtom() {

		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(228).isBackboneAtom());
			assertTrue(s.getAtomByIndex(658).isBackboneAtom());
			assertTrue(s.getAtomByIndex(1003).isBackboneAtom());
			
			assertTrue(!s.getAtomByIndex(526).isBackboneAtom());	
			assertTrue(!s.getAtomByIndex(846).isBackboneAtom());	
			assertTrue(!s.getAtomByIndex(1150).isBackboneAtom());
			assertTrue(!s.getAtomByIndex(1441).isBackboneAtom());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testIsMainChainAtom() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(228).isMainChainAtom());
			assertTrue(s.getAtomByIndex(658).isMainChainAtom());
			assertTrue(s.getAtomByIndex(1003).isMainChainAtom());
			assertTrue(s.getAtomByIndex(1444).isMainChainAtom());
			
			assertTrue(!s.getAtomByIndex(524).isMainChainAtom());	
			assertTrue(!s.getAtomByIndex(846).isMainChainAtom());	
			assertTrue(!s.getAtomByIndex(1150).isMainChainAtom());
			assertTrue(!s.getAtomByIndex(1441).isMainChainAtom());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}
	
	@Test
	public void testIsMainChainHydrogenAtom() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(347).isMainChainHydrogenAtom());
			assertTrue(s.getAtomByIndex(436).isMainChainHydrogenAtom());
			assertTrue(s.getAtomByIndex(530).isMainChainHydrogenAtom());
			
			assertTrue(!s.getAtomByIndex(594).isMainChainHydrogenAtom());	
			assertTrue(!s.getAtomByIndex(657).isMainChainHydrogenAtom());	
			assertTrue(!s.getAtomByIndex(736).isMainChainHydrogenAtom());
			assertTrue(!s.getAtomByIndex(805).isMainChainHydrogenAtom());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	
	@Test
	public void testIsMainChainPlusHAtom() {
	
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(5).isMainChainPlusHAtom());
			assertTrue(s.getAtomByIndex(150).isMainChainPlusHAtom());
			assertTrue(s.getAtomByIndex(228).isMainChainPlusHAtom());
			assertTrue(s.getAtomByIndex(266).isMainChainPlusHAtom());
			assertTrue(s.getAtomByIndex(658).isMainChainPlusHAtom());
			assertTrue(s.getAtomByIndex(1003).isMainChainPlusHAtom());
			assertTrue(s.getAtomByIndex(1444).isMainChainPlusHAtom());
			
			assertTrue(!s.getAtomByIndex(497).isMainChainPlusHAtom());
			assertTrue(!s.getAtomByIndex(524).isMainChainPlusHAtom());
			assertTrue(!s.getAtomByIndex(846).isMainChainPlusHAtom());
			assertTrue(!s.getAtomByIndex(1008).isMainChainPlusHAtom());
			assertTrue(!s.getAtomByIndex(1150).isMainChainPlusHAtom());	
			assertTrue(!s.getAtomByIndex(1441).isMainChainPlusHAtom());	
			
			
			} catch (IOException e) {
				fail(e.toString());
			}
		
	}

	@Test
	public void testIsMainChainPlusCbAtom() {

		try {
		
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(228).isMainChainPlusCbAtom());
			assertTrue(s.getAtomByIndex(507).isMainChainPlusCbAtom());
			assertTrue(s.getAtomByIndex(658).isMainChainPlusCbAtom());
			assertTrue(s.getAtomByIndex(1003).isMainChainPlusCbAtom());
			assertTrue(s.getAtomByIndex(1444).isMainChainPlusCbAtom());
		
			assertTrue(!s.getAtomByIndex(524).isMainChainPlusCbAtom());	
			assertTrue(!s.getAtomByIndex(846).isMainChainPlusCbAtom());	
			assertTrue(!s.getAtomByIndex(1151).isMainChainPlusCbAtom());
			assertTrue(!s.getAtomByIndex(1441).isMainChainPlusCbAtom());
	
			
			} catch (IOException e) {
				fail(e.toString());
			}
		
	}

	@Test
	public void testIsSideChainAtom() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(284).isSideChainAtom());
			assertTrue(s.getAtomByIndex(569).isSideChainAtom());
			assertTrue(s.getAtomByIndex(851).isSideChainAtom());
			assertTrue(s.getAtomByIndex(1151).isSideChainAtom());
		
			assertTrue(!s.getAtomByIndex(291).isSideChainAtom());	
			assertTrue(!s.getAtomByIndex(396).isSideChainAtom());	
			assertTrue(!s.getAtomByIndex(433).isSideChainAtom());
			assertTrue(!s.getAtomByIndex(790).isSideChainAtom());
	
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testIsSideChainMinusHAtom() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
						
			assertTrue(s.getAtomByIndex(569).isSideChainMinusHAtom());
			assertTrue(s.getAtomByIndex(851).isSideChainMinusHAtom());
			assertTrue(s.getAtomByIndex(857).isSideChainMinusHAtom());
			assertTrue(s.getAtomByIndex(1007).isSideChainMinusHAtom());
		
			assertTrue(!s.getAtomByIndex(284).isSideChainMinusHAtom());	
			assertTrue(!s.getAtomByIndex(921).isSideChainMinusHAtom());	
			assertTrue(!s.getAtomByIndex(1151).isSideChainMinusHAtom());
			assertTrue(!s.getAtomByIndex(1222).isSideChainMinusHAtom());
	
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testDistance() {
		
		Atom x = new Atom();
		x.setCoordinates(new Point3D(1,1,1));
		
		Atom y = new Atom();
		y.setCoordinates(new Point3D(6,7,8));
	
		assertEquals(10.488088481702, x.distance(y), 0.0000001);
	}
	
	@Test
	public void testCloneEquals() {
		

		Atom x = new Atom();
		x.setCoordinates(new Point3D(1,1,1));
		
		Atom cx = (Atom)x.clone();
		
		assertTrue(x.equals(cx));
		assertTrue(x!=cx);
		
		x.setCoordinates(new Point3D(2,2,2));
		assertTrue(!x.equals(cx));
		
	}

}
