/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


import java.io.IOException;
import java.util.ArrayList;

import jama.Matrix;
import jgromacs.analysis.Distances;
import jgromacs.data.Atom;
import jgromacs.data.FrameIndexSet;
import jgromacs.data.IndexSet;
import jgromacs.data.Point3D;
import jgromacs.data.PointList;
import jgromacs.data.Residue;
import jgromacs.data.Structure;
import jgromacs.data.Trajectory;
import jgromacs.db.AtomType;
import jgromacs.io.IOData;
import junit.framework.TestCase;

import org.junit.Test;

public class DistancesTest extends TestCase{

	@Test
	public void testGetDistanceMatrix() {
		PointList pl = new PointList();
		pl.addPoint(new Point3D(1,1,1));
		pl.addPoint(new Point3D(3,3,3));
		pl.addPoint(new Point3D(2,4,6));
	
		Matrix M = Distances.getDistanceMatrix(pl);
		
		assertEquals(3.4641016151378, M.get(0,1), 0.0000001);
		assertEquals(3.4641016151378, M.get(1,0), 0.0000001);
		assertEquals(5.9160797830996, M.get(0,2), 0.0000001);
		assertEquals(5.9160797830996, M.get(2,0), 0.0000001);
		assertEquals(3.3166247903554, M.get(1,2), 0.0000001);
		assertEquals(3.3166247903554, M.get(2,1), 0.0000001);
		assertEquals(0, M.get(0,0), 0.0000001);
		assertEquals(0, M.get(1,1), 0.0000001);
		assertEquals(0, M.get(2,2), 0.0000001);
		
	}

	@Test
	public void testGetAtomicDistanceMatrixStructure() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setCoordinates(new Point3D(1,1,1));
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setCoordinates(new Point3D(3,3,3));
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setCoordinates(new Point3D(2,4,6));
		r.addAtom(a3);
		s.addResidue(r);
		
		Matrix M = Distances.getAtomicDistanceMatrix(s);
		
		assertEquals(3.4641016151378, M.get(0,1), 0.0000001);
		assertEquals(3.4641016151378, M.get(1,0), 0.0000001);
		assertEquals(5.9160797830996, M.get(0,2), 0.0000001);
		assertEquals(5.9160797830996, M.get(2,0), 0.0000001);
		assertEquals(3.3166247903554, M.get(1,2), 0.0000001);
		assertEquals(3.3166247903554, M.get(2,1), 0.0000001);
		assertEquals(0, M.get(0,0), 0.0000001);
		assertEquals(0, M.get(1,1), 0.0000001);
		assertEquals(0, M.get(2,2), 0.0000001);
	
	}

	@Test
	public void testGetAtomicDistanceMatrixStructureIndexSet() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setCoordinates(new Point3D(1,1,1));
		a1.setIndex(1);
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setCoordinates(new Point3D(3,3,3));
		a2.setIndex(2);
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setCoordinates(new Point3D(2,4,6));
		a3.setIndex(3);
		r.addAtom(a3);
		Atom a4 = new Atom();
		a4.setCoordinates(new Point3D(7,7,7));
		a4.setIndex(4);
		r.addAtom(a4);
		Atom a5 = new Atom();
		a5.setCoordinates(new Point3D(9,9,9));
		a5.setIndex(5);
		r.addAtom(a5);
		s.addResidue(r);
		
		IndexSet iset = new IndexSet();
		iset.addIndex(1);
		iset.addIndex(2);
		iset.addIndex(3);
		
		Matrix M = Distances.getAtomicDistanceMatrix(s,iset);
		
		assertEquals(3, M.getRowDimension());
		assertEquals(3, M.getColumnDimension());
		
		assertEquals(3.4641016151378, M.get(0,1), 0.0000001);
		assertEquals(3.4641016151378, M.get(1,0), 0.0000001);
		assertEquals(5.9160797830996, M.get(0,2), 0.0000001);
		assertEquals(5.9160797830996, M.get(2,0), 0.0000001);
		assertEquals(3.3166247903554, M.get(1,2), 0.0000001);
		assertEquals(3.3166247903554, M.get(2,1), 0.0000001);
		assertEquals(0, M.get(0,0), 0.0000001);
		assertEquals(0, M.get(1,1), 0.0000001);
		assertEquals(0, M.get(2,2), 0.0000001);
	
	}

	@Test
	public void testGetAtomicDistanceMatrixStructureIndexSetIndexSet() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setCoordinates(new Point3D(1,1,1));
		a1.setIndex(1);
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setCoordinates(new Point3D(3,3,3));
		a2.setIndex(2);
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setCoordinates(new Point3D(2,4,6));
		a3.setIndex(3);
		r.addAtom(a3);
		Atom a4 = new Atom();
		a4.setCoordinates(new Point3D(7,7,7));
		a4.setIndex(4);
		r.addAtom(a4);
		Atom a5 = new Atom();
		a5.setCoordinates(new Point3D(9,9,9));
		a5.setIndex(5);
		r.addAtom(a5);
		s.addResidue(r);
		
		IndexSet iset1 = new IndexSet();
		iset1.addIndex(1);
		iset1.addIndex(2);
		iset1.addIndex(3);
		
		IndexSet iset2 = new IndexSet();
		iset2.addIndex(4);
		iset2.addIndex(5);
		
		Matrix M = Distances.getAtomicDistanceMatrix(s, iset1, iset2);
		
		assertEquals(3, M.getRowDimension());
		assertEquals(2, M.getColumnDimension());
		
		assertEquals(10.392304845413, M.get(0,0), 0.0000001);
		assertEquals(13.856406460551, M.get(0,1), 0.0000001);		
		assertEquals(6.9282032302755, M.get(1,0), 0.0000001);
		assertEquals(10.392304845413, M.get(1,1), 0.0000001);
		assertEquals(5.9160797830996, M.get(2,0), 0.0000001);
		assertEquals(9.1104335791443, M.get(2,1), 0.0000001);
		
	}

	@Test
	public void testGetResidueDistanceMatrix() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Matrix M = Distances.getResidueDistanceMatrix(s, Distances.ALPHACARBON);
			
			assertEquals(0,M.get(0, 0),0.0001);
			assertEquals(0,M.get(19, 19),0.0001);
			assertEquals(0,M.get(56, 56),0.0001);
			
			assertEquals(0.9222337,M.get(5, 59),0.0001);
			assertEquals(1.3430406,M.get(9, 41),0.0001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		
		
	}

	@Test
	public void testGetMeanAtomicDistanceMatrixTrajectory() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		Atom a2 = new Atom();
		Atom a3 = new Atom();
		r.addAtom(a1);
		r.addAtom(a2);
		r.addAtom(a3);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(3,3,3));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(6,6,6));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		
		Matrix M = Distances.getMeanAtomicDistanceMatrix(t);
		
		assertEquals(0,M.get(0, 0),0.0001);
		assertEquals(0,M.get(1, 1),0.0001);
		assertEquals(0,M.get(2, 2),0.0001);
	
		assertEquals(2.5980762,M.get(0, 1),0.0001);
		assertEquals(2.5980762,M.get(1, 2),0.0001);
	}

	@Test
	public void testGetMeanAtomicDistanceMatrixTrajectoryIndexSet() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		Atom a2 = new Atom();
		Atom a3 = new Atom();
		Atom a4 = new Atom();
		Atom a5 = new Atom();
		a1.setIndex(1);
		a2.setIndex(2);
		a3.setIndex(3);
		a4.setIndex(4);
		a5.setIndex(5);
		r.addAtom(a1);
		r.addAtom(a2);
		r.addAtom(a3);
		r.addAtom(a4);
		r.addAtom(a5);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(3,3,3));
		frame1.addPoint(new Point3D(4,4,4));
		frame1.addPoint(new Point3D(5,5,5));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(6,6,6));
		frame2.addPoint(new Point3D(8,8,8));
		frame2.addPoint(new Point3D(10,10,10));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		
		IndexSet iset = new IndexSet();
		iset.addIndex(1);
		iset.addIndex(2);
		iset.addIndex(3);
		
		Matrix M = Distances.getMeanAtomicDistanceMatrix(t,iset);
		
		assertEquals(0,M.get(0, 0),0.0001);
		assertEquals(0,M.get(1, 1),0.0001);
		assertEquals(0,M.get(2, 2),0.0001);
	
		assertEquals(2.5980762,M.get(0, 1),0.0001);
		assertEquals(2.5980762,M.get(1, 2),0.0001);

	}

	@Test
	public void testGetMeanResidueDistanceMatrix() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet fset = new FrameIndexSet();
			fset.addFrame(0);
			fset.addFrame(1);
			
			t = t.getSubTrajectory(fset);
			
			Matrix M = Distances.getMeanResidueDistanceMatrix(t, Distances.ALPHACARBON);
			
			assertEquals(0,M.get(0, 0),0.0001);
			assertEquals(0,M.get(19, 19),0.0001);
			assertEquals(0,M.get(56, 56),0.0001);
			
			assertEquals(1.63446765,M.get(6, 40),0.0001);
			assertEquals(1.5530941,M.get(40, 70),0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		
		
	}

	@Test
	public void testGetContactMatrixPointListDouble() {
	
		PointList pl = new PointList();
		pl.addPoint(new Point3D(1,1,1));
		pl.addPoint(new Point3D(2,2,2));
		pl.addPoint(new Point3D(3,3,3));
		
		Matrix M = Distances.getContactMatrix(pl, 2.0);
	
		assertEquals(1, M.get(0, 0),0.0001);
		assertEquals(1, M.get(1, 1),0.0001);
		assertEquals(1, M.get(2, 2),0.0001);
		
		assertEquals(1, M.get(0, 1),0.0001);
		assertEquals(1, M.get(1, 2),0.0001);
		assertEquals(0, M.get(0, 2),0.0001);
	}

	@Test
	public void testGetContactMatrixStructureIntDouble() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Matrix M = Distances.getContactMatrix(s, Distances.ALPHACARBON,1.0);
			
			assertEquals(1,M.get(0, 0),0.0001);
			assertEquals(1,M.get(19, 19),0.0001);
			assertEquals(1,M.get(56, 56),0.0001);
			
			assertEquals(1, M.get(5, 59),0.0001);
			assertEquals(0, M.get(9, 41),0.0001);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetContactOfMeanMatrix() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet fset = new FrameIndexSet();
			fset.addFrame(0);
			fset.addFrame(1);
			
			t = t.getSubTrajectory(fset);
			
			Matrix M = Distances.getContactOfMeanMatrix(t, Distances.ALPHACARBON,1.6);
			
			assertEquals(1,M.get(0, 0),0.0001);
			assertEquals(1,M.get(19, 19),0.0001);
			assertEquals(1,M.get(56, 56),0.0001);
			
			assertEquals(0,M.get(6, 40),0.0001);
			assertEquals(1,M.get(40, 70),0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		
		
	}

	@Test
	public void testGetFrequencyContactMatrix() {
		Structure s = new Structure();
		
		Residue r1 = new Residue();
		Atom a1 = new Atom();
		a1.setName("CA");
		a1.setAtomType(new AtomType("C"));
		r1.addAtom(a1);
		s.addResidue(r1);
		
		Residue r2 = new Residue();
		Atom a2 = new Atom();
		a2.setName("CA");
		a2.setAtomType(new AtomType("C"));
		r2.addAtom(a2);
		s.addResidue(r2);
		
		Residue r3 = new Residue();
		Atom a3 = new Atom();
		a3.setName("CA");
		a3.setAtomType(new AtomType("C"));
		r3.addAtom(a3);
		s.addResidue(r3);
		
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(3,3,3));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(6,6,6));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(3,3,3));
		frame3.addPoint(new Point3D(6,6,6));
		frame3.addPoint(new Point3D(9,9,9));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		
		Matrix M1 = Distances.getFrequencyContactMatrix(t, Distances.ALPHACARBON, 4.0, 0.7);
		Matrix M2 = Distances.getFrequencyContactMatrix(t, Distances.ALPHACARBON, 4.0, 0.6);
		Matrix M3 = Distances.getFrequencyContactMatrix(t, Distances.ALPHACARBON, 4.0, 0.15);
		Matrix M4 = Distances.getFrequencyContactMatrix(t, Distances.ALPHACARBON, 4.0, 0.94);
		
		assertEquals(1, M1.get(0, 0),0.00001);
		assertEquals(1, M1.get(1, 1),0.00001);
		assertEquals(1, M1.get(2, 2),0.00001);
	
		assertEquals(0, M1.get(0, 1),0.00001);
		assertEquals(1, M2.get(0, 1),0.00001);
		assertEquals(1, M3.get(0, 1),0.00001);
		assertEquals(0, M4.get(0, 1),0.00001);
	}

	@Test
	public void testGetDistanceTimeSeriesTrajectoryIntInt() {

		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		Atom a2 = new Atom();
		Atom a3 = new Atom();
		a1.setIndex(1);
		a2.setIndex(2);
		a3.setIndex(3);
		r.addAtom(a1);
		r.addAtom(a2);
		r.addAtom(a3);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(3,3,3));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(6,6,6));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(3,3,3));
		frame3.addPoint(new Point3D(6,6,6));
		frame3.addPoint(new Point3D(9,9,9));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		
		ArrayList<Double> result = Distances.getDistanceTimeSeries(t, 1, 2);	
		
		assertEquals(1.7320508, result.get(0), 0.0001);
		assertEquals(3.4641016, result.get(1), 0.0001);
		assertEquals(5.196152, result.get(2), 0.0001);
	}

	@Test
	public void testGetDistanceTimeSeriesTrajectoryIntIndexSet() {

		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		r.addAtom(a3);
		Atom a4 = new Atom();
		a4.setIndex(4);
		r.addAtom(a4);
		Atom a5 = new Atom();
		a5.setIndex(5);
		r.addAtom(a5);
		s.addResidue(r);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(5,5,5));
		frame1.addPoint(new Point3D(1.1,1.1,1.1));
		frame1.addPoint(new Point3D(5.1,5.1,5.1));
		frame1.addPoint(new Point3D(10,10,10));
	
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(1,1,1));
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(3,3,3));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(5,5,5));
	
		
		Trajectory t = new Trajectory();
		t.setStructure(s);
		t.addFrame(frame1);
		t.addFrame(frame2);
		
		IndexSet iset = new IndexSet();
		iset.addIndex(2);
		iset.addIndex(3);
		iset.addIndex(4);
		
		ArrayList<Double> result = Distances.getDistanceTimeSeries(t, 1, iset);
		
		assertEquals(0.1732050807, result.get(0),0.0000001);
		assertEquals(1.7320508075, result.get(1),0.0000001);
	}

	@Test
	public void testGetDistanceTimeSeriesTrajectoryIndexSetIndexSet() {

		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		r.addAtom(a3);
		Atom a4 = new Atom();
		a4.setIndex(4);
		r.addAtom(a4);
		Atom a5 = new Atom();
		a5.setIndex(5);
		r.addAtom(a5);
		s.addResidue(r);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(5,5,5));
		frame1.addPoint(new Point3D(1.5,1.5,1.5));
		frame1.addPoint(new Point3D(3.2,3.2,3.2));
		frame1.addPoint(new Point3D(-1,1,1));
	
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(1,1,1));
		frame2.addPoint(new Point3D(2.1,-2,2));
		frame2.addPoint(new Point3D(3,3,3));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(5,5,5));
	
		
		Trajectory t = new Trajectory();
		t.setStructure(s);
		t.addFrame(frame1);
		t.addFrame(frame2);
		
		IndexSet iset1 = new IndexSet();
		iset1.addIndex(1);
		iset1.addIndex(2);
		iset1.addIndex(3);
		
		IndexSet iset2 = new IndexSet();
		iset2.addIndex(4);
		iset2.addIndex(5);
		
		ArrayList<Double> result = Distances.getDistanceTimeSeries(t, iset1, iset2);
		
		assertEquals(2.000, result.get(0),0.0000001);
		assertEquals(1.7320508075, result.get(1),0.0000001);
	}

	@Test
	public void testGetMeanDistance() {

		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		Atom a2 = new Atom();
		Atom a3 = new Atom();
		a1.setIndex(1);
		a2.setIndex(2);
		a3.setIndex(3);
		r.addAtom(a1);
		r.addAtom(a2);
		r.addAtom(a3);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(3,3,3));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(6,6,6));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(3,3,3));
		frame3.addPoint(new Point3D(6,6,6));
		frame3.addPoint(new Point3D(9,9,9));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		
		assertEquals(3.464101467, Distances.getMeanDistance(t, 1, 2),0.00001);
		
	}

	@Test
	public void testGetVarianceOfDistance() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		Atom a2 = new Atom();
		Atom a3 = new Atom();
		a1.setIndex(1);
		a2.setIndex(2);
		a3.setIndex(3);
		r.addAtom(a1);
		r.addAtom(a2);
		r.addAtom(a3);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(3,3,3));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(6,6,6));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(3,3,3));
		frame3.addPoint(new Point3D(6,6,6));
		frame3.addPoint(new Point3D(9,9,9));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		
		assertEquals(1.99999952, Distances.getVarianceOfDistance(t, 1, 2),0.00001);
	
	}

	@Test
	public void testGetMinimalDistance() {

		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		Atom a2 = new Atom();
		Atom a3 = new Atom();
		a1.setIndex(1);
		a2.setIndex(2);
		a3.setIndex(3);
		r.addAtom(a1);
		r.addAtom(a2);
		r.addAtom(a3);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(3,3,3));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(6,6,6));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(3,3,3));
		frame3.addPoint(new Point3D(6,6,6));
		frame3.addPoint(new Point3D(9,9,9));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		
		assertEquals(1.7320508, Distances.getMinimalDistance(t, 1, 2),0.00001);
		

	}

	@Test
	public void testGetMaximalDistance() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		Atom a2 = new Atom();
		Atom a3 = new Atom();
		a1.setIndex(1);
		a2.setIndex(2);
		a3.setIndex(3);
		r.addAtom(a1);
		r.addAtom(a2);
		r.addAtom(a3);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(3,3,3));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(6,6,6));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(3,3,3));
		frame3.addPoint(new Point3D(6,6,6));
		frame3.addPoint(new Point3D(9,9,9));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		
		assertEquals(5.196152, Distances.getMaximalDistance(t, 1, 2),0.00001);
		

	}

	@Test
	public void testGetDistanceRange() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		Atom a2 = new Atom();
		Atom a3 = new Atom();
		a1.setIndex(1);
		a2.setIndex(2);
		a3.setIndex(3);
		r.addAtom(a1);
		r.addAtom(a2);
		r.addAtom(a3);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(3,3,3));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(6,6,6));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(3,3,3));
		frame3.addPoint(new Point3D(6,6,6));
		frame3.addPoint(new Point3D(9,9,9));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		
		assertEquals(3.4641012, Distances.getDistanceRange(t, 1, 2),0.00001);
		

	}

	@Test
	public void testGetAtomIndicesInRadiusStructurePoint3DDouble() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		a1.setCoordinates(new Point3D(1,1,1));
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		a2.setCoordinates(new Point3D(3,3,3));
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		a3.setCoordinates(new Point3D(2,4,6));
		r.addAtom(a3);
		s.addResidue(r);
		
		IndexSet result = Distances.getAtomIndicesInRadius(s, new Point3D(0.5,0.5,0.5), 0.9);
		IndexSet control = new IndexSet();
		control.addIndex(1);
		assertEquals(control, result);
		
		result = Distances.getAtomIndicesInRadius(s, new Point3D(0.5,0.5,0.5), 4.5);
		control = new IndexSet();
		control.addIndex(1);
		control.addIndex(2);
		assertEquals(control, result);
	
		
	}

	@Test
	public void testGetAtomIndicesInRadiusStructureAtomDouble() {
	
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		a1.setCoordinates(new Point3D(1,1,1));
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		a2.setCoordinates(new Point3D(3,3,3));
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		a3.setCoordinates(new Point3D(2,4,6));
		r.addAtom(a3);
		s.addResidue(r);
		
		IndexSet result = Distances.getAtomIndicesInRadius(s, a1, 3.5);
		IndexSet control = new IndexSet();
		control.addIndex(1);
		control.addIndex(2);
		assertEquals(control, result);
		
		result = Distances.getAtomIndicesInRadius(s, a1, 6);
		control = new IndexSet();
		control.addIndex(1);
		control.addIndex(2);
		control.addIndex(3);
		assertEquals(control, result);

	}

	@Test
	public void testGetAtomIndicesInRadiusStructureIndexSetDouble() {
		
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		a1.setCoordinates(new Point3D(1,1,1));
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		a2.setCoordinates(new Point3D(5,5,5));
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		a3.setCoordinates(new Point3D(1.1,1.1,1.1));
		r.addAtom(a3);
		Atom a4 = new Atom();
		a4.setIndex(4);
		a4.setCoordinates(new Point3D(5.1,5.1,5.1));
		r.addAtom(a4);
		Atom a5 = new Atom();
		a5.setIndex(5);
		a5.setCoordinates(new Point3D(10,10,10));
		r.addAtom(a5);
		s.addResidue(r);
		
		
		IndexSet iset = new IndexSet();
		iset.addIndex(1);
		iset.addIndex(2);
		
		IndexSet result = Distances.getAtomIndicesInRadius(s, iset, 0.5);
		
		IndexSet control = new IndexSet();
		control.addIndex(1);
		control.addIndex(2);
		control.addIndex(3);
		control.addIndex(4);
		
		assertEquals(control, result);
		
	}

	@Test
	public void testGetAtomIndicesInRadiusStructureIndexSetIndexSetDouble() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		a1.setCoordinates(new Point3D(1,1,1));
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		a2.setCoordinates(new Point3D(5,5,5));
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		a3.setCoordinates(new Point3D(1.1,1.1,1.1));
		r.addAtom(a3);
		Atom a4 = new Atom();
		a4.setIndex(4);
		a4.setCoordinates(new Point3D(5.1,5.1,5.1));
		r.addAtom(a4);
		Atom a5 = new Atom();
		a5.setIndex(5);
		a5.setCoordinates(new Point3D(10,10,10));
		r.addAtom(a5);
		s.addResidue(r);
		
		
		IndexSet iset = new IndexSet();
		iset.addIndex(1);
		iset.addIndex(2);
		
		IndexSet iset2 = new IndexSet();
		iset2.addIndex(2);
		iset2.addIndex(3);
		iset2.addIndex(5);
		
		IndexSet result = Distances.getAtomIndicesInRadius(s, iset2, iset, 0.5);
		
		IndexSet control = new IndexSet();
		control.addIndex(2);
		control.addIndex(3);
		
		assertEquals(control, result);
	}

	@Test
	public void testFindFrameWhereClosest() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		r.addAtom(a3);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(5,5,5));
		frame1.addPoint(new Point3D(1,1,1));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(1,1,1));
		frame2.addPoint(new Point3D(3,3,3));
		frame2.addPoint(new Point3D(2,2,2));

		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(1,1,1));
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(3,3,3));
		
		PointList frame4 = new PointList();
		frame4.addPoint(new Point3D(1,1,1));
		frame4.addPoint(new Point3D(4,5,4));
		frame4.addPoint(new Point3D(4,4,4));

		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		t.addFrame(frame4);
		
		assertEquals(frame3, Distances.findFrameWhereClosest(t, 1, 2));
	}

	@Test
	public void testFindFrameWhereMostDistant() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		r.addAtom(a3);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(5,5,5));
		frame1.addPoint(new Point3D(1,1,1));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(1,1,1));
		frame2.addPoint(new Point3D(3,3,3));
		frame2.addPoint(new Point3D(2,2,2));

		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(1,1,1));
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(3,3,3));
		
		PointList frame4 = new PointList();
		frame4.addPoint(new Point3D(1,1,1));
		frame4.addPoint(new Point3D(4,5,4));
		frame4.addPoint(new Point3D(4,4,4));

		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		t.addFrame(frame4);
		
		assertEquals(frame1, Distances.findFrameWhereMostDistant(t, 1, 2));

	}

	@Test
	public void testGetDistanceOfAtomToAtomSet() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		a1.setCoordinates(new Point3D(1,1,1));
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		a2.setCoordinates(new Point3D(5,5,5));
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		a3.setCoordinates(new Point3D(1.1,1.1,1.1));
		r.addAtom(a3);
		Atom a4 = new Atom();
		a4.setIndex(4);
		a4.setCoordinates(new Point3D(5.1,5.1,5.1));
		r.addAtom(a4);
		Atom a5 = new Atom();
		a5.setIndex(5);
		a5.setCoordinates(new Point3D(10,10,10));
		r.addAtom(a5);
		s.addResidue(r);
		
		IndexSet iset = new IndexSet();
		iset.addIndex(2);
		iset.addIndex(3);
		iset.addIndex(4);
				
		assertEquals(0.1732050807, Distances.getDistanceOfAtomToAtomSet(s, 1, iset), 0.0000001);
		
	}

	@Test
	public void testGetDistanceOfTwoAtomSets() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		a1.setCoordinates(new Point3D(1,1,1));
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		a2.setCoordinates(new Point3D(2,2,2));
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		a3.setCoordinates(new Point3D(3,3,3));
		r.addAtom(a3);
		Atom a4 = new Atom();
		a4.setIndex(4);
		a4.setCoordinates(new Point3D(4,4,4));
		r.addAtom(a4);
		Atom a5 = new Atom();
		a5.setIndex(5);
		a5.setCoordinates(new Point3D(5,5,5));
		r.addAtom(a5);
		s.addResidue(r);
		
		IndexSet iset1 = new IndexSet();
		iset1.addIndex(1);
		iset1.addIndex(2);
		iset1.addIndex(3);

		IndexSet iset2 = new IndexSet();
		iset2.addIndex(4);
		iset2.addIndex(5);
		
		assertEquals(1.732050807568, Distances.getDistanceOfTwoAtomSets(s, iset1, iset2), 0.0000001);
		
	}

	@Test
	public void testFindClosestAtom() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		a1.setCoordinates(new Point3D(1,1,1));
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		a2.setCoordinates(new Point3D(5,5,5));
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		a3.setCoordinates(new Point3D(1.1,1.1,1.1));
		r.addAtom(a3);
		Atom a4 = new Atom();
		a4.setIndex(4);
		a4.setCoordinates(new Point3D(5.1,5.1,5.1));
		r.addAtom(a4);
		Atom a5 = new Atom();
		a5.setIndex(5);
		a5.setCoordinates(new Point3D(10,10,10));
		r.addAtom(a5);
		Atom a6 = new Atom();
		a6.setIndex(6);
		a6.setCoordinates(new Point3D(11,11,11));
		r.addAtom(a6);
		s.addResidue(r);
		
		IndexSet iset1 = new IndexSet();
		iset1.addIndex(1);
		iset1.addIndex(2);
		iset1.addIndex(4);
		
		IndexSet iset2 = new IndexSet();
		iset2.addIndex(3);
		iset2.addIndex(5);
		iset2.addIndex(6);
		
		IndexSet iset3 = new IndexSet();
		iset3.addIndex(1);
		iset3.addIndex(2);
		iset3.addIndex(3);
		
		IndexSet iset4 = new IndexSet();
		iset4.addIndex(4);
		iset4.addIndex(5);
		iset4.addIndex(6);
		
		assertEquals(a1, Distances.findClosestAtom(s, iset1, iset2));		
		assertEquals(a2, Distances.findClosestAtom(s, iset3, iset4));
	}

	@Test
	public void testGetFramesWhereAtomsAreClose() {
	
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		r.addAtom(a3);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(5,5,5));
		frame1.addPoint(new Point3D(1,1,1));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(1,1,1));
		frame2.addPoint(new Point3D(3,3,3));
		frame2.addPoint(new Point3D(2,2,2));

		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(1,1,1));
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(3,3,3));
		
		PointList frame4 = new PointList();
		frame4.addPoint(new Point3D(1,1,1));
		frame4.addPoint(new Point3D(4,5,4));
		frame4.addPoint(new Point3D(4,4,4));

		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		t.addFrame(frame4);
		
		FrameIndexSet result = Distances.getFramesWhereAtomsAreClose(t, 1, 2, 3.5);
		result.setName("");
		
		FrameIndexSet control = new FrameIndexSet("");
		control.addFrame(1);
		control.addFrame(2);
		
		assertEquals(control, result);
	}

	@Test
	public void testGetFramesWhereAtomsAreDistant() {

		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		r.addAtom(a3);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(5,5,5));
		frame1.addPoint(new Point3D(1,1,1));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(1,1,1));
		frame2.addPoint(new Point3D(3,3,3));
		frame2.addPoint(new Point3D(2,2,2));

		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(1,1,1));
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(3,3,3));
		
		PointList frame4 = new PointList();
		frame4.addPoint(new Point3D(1,1,1));
		frame4.addPoint(new Point3D(4,5,4));
		frame4.addPoint(new Point3D(4,4,4));

		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		t.addFrame(frame4);
		
		FrameIndexSet result = Distances.getFramesWhereAtomsAreDistant(t, 1, 2, 3.5);
		result.setName("");
		
		FrameIndexSet control = new FrameIndexSet("");
		control.addFrame(0);
		control.addFrame(3);
		
		assertEquals(control, result);

	}

	@Test
	public void testGetFramesWhereAtomIsCloseToPoint() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		r.addAtom(a3);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(5,5,5));
		frame1.addPoint(new Point3D(1,1,1));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(1,1,1));
		frame2.addPoint(new Point3D(3,3,3));
		frame2.addPoint(new Point3D(2,2,2));

		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(1,1,1));
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(3,3,3));
		
		PointList frame4 = new PointList();
		frame4.addPoint(new Point3D(1,1,1));
		frame4.addPoint(new Point3D(4,5,4));
		frame4.addPoint(new Point3D(4,4,4));

		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		t.addFrame(frame4);
		
		FrameIndexSet result = Distances.getFramesWhereAtomIsCloseToPoint(t, 2, new Point3D(5,5,4), 1.1);
		result.setName("");
		
		FrameIndexSet control = new FrameIndexSet("");
		control.addFrame(0);
		control.addFrame(3);
		
		assertEquals(control, result);

	}

	@Test
	public void testGetFramesWhereAtomIsDistantFromPoint() {
		
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		r.addAtom(a3);
		s.addResidue(r);
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(5,5,5));
		frame1.addPoint(new Point3D(1,1,1));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(1,1,1));
		frame2.addPoint(new Point3D(3,3,3));
		frame2.addPoint(new Point3D(2,2,2));

		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(1,1,1));
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(3,3,3));
		
		PointList frame4 = new PointList();
		frame4.addPoint(new Point3D(1,1,1));
		frame4.addPoint(new Point3D(4,5,4));
		frame4.addPoint(new Point3D(4,4,4));

		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		t.addFrame(frame4);
		
		FrameIndexSet result = Distances.getFramesWhereAtomIsDistantFromPoint(t, 2, new Point3D(5,5,4), 1.1);
		result.setName("");
		
		FrameIndexSet control = new FrameIndexSet("");
		control.addFrame(1);
		control.addFrame(2);
		
		assertEquals(control, result);


		
	}

}
