/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


import jama.Matrix;

import java.io.IOException;
import java.util.ArrayList;

import jgromacs.analysis.Distances;
import jgromacs.analysis.Dynamics;
import jgromacs.analysis.Similarity;
import jgromacs.analysis.Superposition;
import jgromacs.data.Atom;
import jgromacs.data.IndexSet;
import jgromacs.data.Point3D;
import jgromacs.data.PointList;
import jgromacs.data.Residue;
import jgromacs.data.Structure;
import jgromacs.data.Trajectory;
import jgromacs.db.AtomType;
import jgromacs.io.IOData;
import junit.framework.TestCase;

import org.junit.Test;

public class DynamicsTest extends TestCase {

	@Test
	public void testGetCoordinateCovarianceMatrixTrajectoryPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			
			Trajectory t = IOData.readDumpedTrajectory(s,"dataset//example_calpha.dxtc");
			
			Matrix M = Dynamics.getCoordinateCovarianceMatrix(t, s.getAllAtomCoordinates());
		
			assertEquals(3*t.getNumberOfAtoms(), M.getRowDimension());
			assertEquals(3*t.getNumberOfAtoms(), M.getColumnDimension());
		
			// control values calculated with the Gromacs command g_covar:
			assertEquals(0.00213723,M.get(0, 0),0.00000001);
			assertEquals(0.000533495,M.get(0, 4),0.00000001);
			assertEquals(0.000367467,M.get(0, 21),0.00000001);
			assertEquals(0.000337971,M.get(0, 48),0.00000001);
		
			
			} catch (IOException e) {
				fail(e.toString());
			}	
	}

	@Test
	public void testGetAtomicCovarianceMatrixTrajectoryPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			Matrix M = Dynamics.getAtomicCovarianceMatrix(t, s.getAllAtomCoordinates());
			
			assertEquals(t.getNumberOfAtoms(), M.getRowDimension());
			assertEquals(t.getNumberOfAtoms(), M.getColumnDimension());
					
			// control values calculated with the Gromacs command g_covar:
			assertEquals(0.00546,M.get(0, 0),0.00005);
			assertEquals(0.00243,M.get(0, 1),0.00005);
			assertEquals(0.00111,M.get(5, 5),0.00005);
			assertEquals(0.00586,M.get(12, 12),0.00005);
		
			
			} catch (IOException e) {
				fail(e.toString());
			}	
	}

	@Test
	public void testGetCoordinateCorrelationMatrixTrajectoryPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			Matrix M = Dynamics.getCoordinateCorrelationMatrix(t, s.getAllAtomCoordinates());
		
			assertEquals(3*t.getNumberOfAtoms(), M.getRowDimension());
			assertEquals(3*t.getNumberOfAtoms(), M.getColumnDimension());
		
			// control values calculated with the Gromacs command g_covar (transformed to correlation values):
			assertEquals(1,M.get(0, 0),0.00000001);
			assertEquals(1,M.get(4, 4),0.00000001);
			assertEquals(1,M.get(19, 19),0.00000001);
			assertEquals(1,M.get(45, 45),0.00000001);
			assertEquals(1,M.get(178, 178),0.00000001);
			
			assertEquals(0.478200159,M.get(0, 4),0.00000001);
			assertEquals(-0.41773464,M.get(9, 48),0.00000001);
			assertEquals(0.327125256,M.get(76, 87),0.00000001);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetAtomicCorrelationMatrixTrajectoryPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			Matrix M = Dynamics.getAtomicCorrelationMatrix(t, s.getAllAtomCoordinates());
		
			assertEquals(t.getNumberOfAtoms(), M.getRowDimension());
			assertEquals(t.getNumberOfAtoms(), M.getColumnDimension());
		
			// control values calculated with the Gromacs command g_covar (transformed to correlation values):
			assertEquals(1,M.get(0, 0),0.00000001);
			assertEquals(1,M.get(4, 4),0.00000001);
			assertEquals(1,M.get(19, 19),0.00000001);
			assertEquals(1,M.get(45, 45),0.00000001);
			assertEquals(1,M.get(78, 78),0.00000001);
			
			assertEquals(-0.24992296,M.get(6, 15),0.00000001);
			assertEquals(0.198114177,M.get(22, 44),0.00000001);
			assertEquals(-0.13557394,M.get(81, 86),0.00000001);
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetPCATrajectory() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			Matrix[] result = Dynamics.getPCA(t);
			
			Matrix D = result[0];
				
			// control values calculated with the Gromacs command g_covar: 
			int n = D.getRowDimension();
			assertEquals(0.0311839, D.get(n-1, n-1), 0.0001);
			assertEquals(0.0252699, D.get(n-2, n-2), 0.0001);
			assertEquals(0.0179765, D.get(n-3, n-3), 0.0001);
			assertEquals(0.0154887, D.get(n-4, n-4), 0.0001);
			assertEquals(0.0135182, D.get(n-5, n-5), 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetCumulativeVariancesTrajectory() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			ArrayList<Double> result = Dynamics.getCumulativeVariances(t);
					
			assertEquals(0.167652, result.get(0), 0.0001);
			assertEquals(0.303509, result.get(1), 0.0001);
			assertEquals(0.400155, result.get(2), 0.0001);	
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetRootMeanSquareInnerProductTrajectoryTrajectoryIntInt() {
	
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			
			Trajectory t1 = IOData.readDumpedTrajectory(s, "dataset//t1.dxtc");
			Trajectory t2 = IOData.readDumpedTrajectory(s, "dataset//t2.dxtc");
			
			double result = Dynamics.getRootMeanSquareInnerProduct(t2, t1, 9, 8);
		
			// Control value are calculated with the Gromacs command g_anaeig:
			assertEquals(0.431, result, 0.001);
						
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetCovarianceMatrixOverlapTrajectoryTrajectory() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			Trajectory t1 = IOData.readDumpedTrajectory(s, "dataset//t1.dxtc");
			Trajectory t2 = IOData.readDumpedTrajectory(s, "dataset//t2.dxtc");
			
			// Control values are calculated with the Gromacs command g_anaeig:
			
			double result = Dynamics.getCovarianceMatrixOverlap(t2,t, 9);
			assertEquals(0.404, result, 0.001);
			
			result = Dynamics.getCovarianceMatrixOverlap(t1,t, 10);
			assertEquals(0.428, result, 0.001);
			
			result = Dynamics.getCovarianceMatrixOverlap(t1,t2, 9);
			assertEquals(0.121, result, 0.001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetFluctuationMatrixTrajectory() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			Matrix M = Dynamics.getFluctuationMatrix(t);
			
			assertEquals(t.getNumberOfAtoms(), M.getRowDimension());
			assertEquals(t.getNumberOfAtoms(), M.getColumnDimension());
		
			assertEquals(0, M.get(5, 5),0.000001);
			assertEquals(0, M.get(24, 24),0.000001);
			assertEquals(0, M.get(66, 66),0.000001);
			
			double f = Distances.getVarianceOfDistance(t, 8, 29);
			assertEquals(f, M.get(7, 28),0.000001);
			
			f = Distances.getVarianceOfDistance(t, 11, 42);
			assertEquals(f, M.get(10, 41),0.000001);
			
			f = Distances.getVarianceOfDistance(t, 7, 66);
			assertEquals(f, M.get(6, 65),0.000001);
			
			f = Distances.getVarianceOfDistance(t, 30, 50);
			assertEquals(f, M.get(29, 49),0.000001);
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetFluctuationMatrixTrajectoryIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
						
			Matrix M = Dynamics.getFluctuationMatrix(t, iset);
			
			assertEquals(3, M.getRowDimension());
			assertEquals(3, M.getColumnDimension());
		
			ArrayList<Integer> indices = iset.getAsArrayList();
			
			for (int i = 0; i < indices.size(); i++) {
				for (int j = 0; j < indices.size(); j++) {
				double f = Distances.getVarianceOfDistance(t, indices.get(i), indices.get(j));
				assertEquals(f, M.get(i, j),0.0000001);
			  }
			}
	
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetFluctationOfSubsets() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			IndexSet iset1 = new IndexSet();
			iset1.addIndex(10);
			iset1.addIndex(20);
			
			IndexSet iset2 = new IndexSet();
			iset2.addIndex(30);
						
			double f = Dynamics.getFluctuationOfSubsets(t, iset1, iset2);
			
			double f1 = Distances.getVarianceOfDistance(t, 10, 30);
			double f2 = Distances.getVarianceOfDistance(t, 20, 30);
			
			assertEquals((f1+f2)/2, f, 0.0000001);
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetRMSFprofileTrajectoryPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			ArrayList<Double> result = Dynamics.getRMSFprofile(t, s.getAllAtomCoordinates());
			
			assertEquals(93, result.size());
			
			// Control RMSF values are calculated with the Gromacs command g_rmsf
			assertEquals(0.0809, result.get(0),0.00005);
			assertEquals(0.0682, result.get(22),0.00005);
			assertEquals(0.0860, result.get(49),0.00005);
			assertEquals(0.0977, result.get(84),0.00005);
			assertEquals(0.0600, result.get(90),0.00005);
			
		} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetRMSFprofileTrajectoryIndexSetStructureIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(3);
			iset.addIndex(20);
			iset.addIndex(42);
			iset.addIndex(50);
			iset.addIndex(71);
			
			ArrayList<Double> result = Dynamics.getRMSFprofile(t, iset, s, iset);
			
			assertEquals(5, result.size());
			
			// Control RMSF values are calculated with the Gromacs command g_rmsf
			assertEquals(0.0370, result.get(0),0.00005);		
			assertEquals(0.0405, result.get(1),0.00005);
			assertEquals(0.0600, result.get(2),0.00005);
			assertEquals(0.0550, result.get(3),0.00005);	
			assertEquals(0.0568, result.get(4),0.00005);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetRMSFprofileTrajectoryPointListPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			ArrayList<Double> result = Dynamics.getRMSFprofile(t, s.getAllAtomCoordinates(), s.getAllAtomCoordinates());
			
			assertEquals(93, result.size());
			
			// Control RMSF values are calculated with the Gromacs command g_rmsf
			assertEquals(0.0809, result.get(0),0.00005);
			assertEquals(0.0682, result.get(22),0.00005);
			assertEquals(0.0860, result.get(49),0.00005);
			assertEquals(0.0977, result.get(84),0.00005);
			assertEquals(0.0600, result.get(90),0.00005);

			
			Trajectory t2 = (Trajectory)t.clone();
			t2 = Superposition.superposeTo(t2, s.getAllAtomCoordinates());
			PointList Q = t2.getMeanFrame();
			result = Dynamics.getRMSFprofile(t, s.getAllAtomCoordinates(), Q);
			
			assertEquals(93, result.size());
			
			// Control RMSF values are calculated with the Gromacs command g_rmsf
			assertEquals(0.0742, result.get(0),0.00005);
			assertEquals(0.0398, result.get(22),0.00005);
			assertEquals(0.0523, result.get(49),0.00005);
			assertEquals(0.0494, result.get(84),0.00005);
			assertEquals(0.0331, result.get(90),0.00005);

			
			
		} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetRMSFaroundTheMeanProfileTrajectoryPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			ArrayList<Double> result = Dynamics.getRMSFaroundTheMeanProfile(t, s.getAllAtomCoordinates());
			
			assertEquals(93, result.size());
			
			// Control RMSF values are calculated with the Gromacs command g_rmsf
			assertEquals(0.0742, result.get(0),0.00005);
			assertEquals(0.0398, result.get(22),0.00005);
			assertEquals(0.0523, result.get(49),0.00005);
			assertEquals(0.0494, result.get(84),0.00005);
			assertEquals(0.0331, result.get(90),0.00005);
			
		} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetDynamicalNetwork() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_protein.dxtc");
			
			Matrix result = Dynamics.getDynamicalNetwork(t, 0.5, 0.8);
			
			assertEquals(s.getNumberOfResidues(), result.getRowDimension());
			assertEquals(s.getNumberOfResidues(), result.getColumnDimension());
			
			Matrix corr = Dynamics.getAtomicCorrelationMatrix(t.getSubTrajectory(s.getAlphaCarbonIndexSet()));
			Matrix contact = Distances.getFrequencyContactMatrix(t, Distances.CLOSESTHEAVY, 0.5, 0.8);
			
			for (int i = 0; i < result.getRowDimension(); i++) {
				for (int j = 0; j < result.getColumnDimension(); j++) {
				
					if (contact.get(i, j)==0) assertEquals(Double.NaN, result.get(i, j),0.00001);
					else {
						double d = -Math.log(Math.abs(corr.get(i, j)));
						assertEquals(d, result.get(i, j), 0.00001);
					}
					
				}
			}
					
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetStructuralRadius() {
		
		Trajectory t = new Trajectory();

		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(3,3,3));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(6,6,6));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(3,3,3));
		frame3.addPoint(new Point3D(6,6,6));
		frame3.addPoint(new Point3D(9,9,9));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);

		double rmsd1 = Similarity.getRMSD(frame1, frame2);
		double rmsd2 = Similarity.getRMSD(frame1, frame3);
		double rmsd3 = Similarity.getRMSD(frame2, frame3);
		double control = Math.sqrt((2*rmsd1*rmsd1+2*rmsd2*rmsd2+2*rmsd3*rmsd3)/(2*9));

		double result = Dynamics.getStructuralRadius(t);
		
		assertEquals(control, result, 0.000001);
	}

	@Test
	public void testGetEnsembleAveragedRMSD() {
		
		Trajectory t1 = new Trajectory();
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(3,3,3));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(6,6,6));
				
		t1.addFrame(frame1);
		t1.addFrame(frame2);
		
		Trajectory t2 = new Trajectory();
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(3,2,-1));
		frame3.addPoint(new Point3D(1,2,4));
		frame3.addPoint(new Point3D(3,2,3));
		
		PointList frame4 = new PointList();
		frame4.addPoint(new Point3D(1,7,5));
		frame4.addPoint(new Point3D(3,4,1));
		frame4.addPoint(new Point3D(6,6,1));
				
		t2.addFrame(frame3);
		t2.addFrame(frame4);

		double result = Dynamics.getEnsembleAveragedRMSD(t1, t2);
		
		double A = Math.pow(Similarity.getRMSD(frame1, frame3),2);
		double B = Math.pow(Similarity.getRMSD(frame1, frame4),2);
		double C = Math.pow(Similarity.getRMSD(frame2, frame3),2);
		double D = Math.pow(Similarity.getRMSD(frame2, frame4),2);

		double control= Math.sqrt((A+B+C+D)/4);
		
		assertEquals(control, result, 0.0001);
		
	}

	@Test
	public void testGetContactProbabilityMap() {
		
		Structure s = new Structure();
		
		Residue r1 = new Residue();
		Atom a1 = new Atom();
		a1.setName("CA");
		a1.setAtomType(new AtomType("C"));
		r1.addAtom(a1);
		s.addResidue(r1);
		
		Residue r2 = new Residue();
		Atom a2 = new Atom();
		a2.setName("CA");
		a2.setAtomType(new AtomType("C"));
		r2.addAtom(a2);
		s.addResidue(r2);
		
		Residue r3 = new Residue();
		Atom a3 = new Atom();
		a3.setName("CA");
		a3.setAtomType(new AtomType("C"));
		r3.addAtom(a3);
		s.addResidue(r3);
		
		Trajectory t = new Trajectory();
		t.setStructure(s);
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(1,1,1));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(3,3,3));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(4,4,4));
		frame2.addPoint(new Point3D(6,6,6));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(3,3,3));
		frame3.addPoint(new Point3D(6,6,6));
		frame3.addPoint(new Point3D(9,9,9));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		
		Matrix result = Dynamics.getContactProbabilityMap(t, 3.5);
		
		assertEquals(1, result.get(0, 0),0.00001);
		assertEquals(0.6666, result.get(0, 1),0.0001);
		assertEquals(0.3333, result.get(0, 2),0.0001);
		assertEquals(0.6666, result.get(1, 2),0.0001);
		
	}

	@Test
	public void testGetTrajectoryOfAtom() {

		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			ArrayList<Point3D> result = Dynamics.getTrajectoryOfAtom(t, 5);
			
			assertEquals(20, result.size());
			
			assertEquals(new Point3D(1.577, 2.522,  3.407), result.get(0));
			assertEquals(new Point3D(1.579, 2.475,  3.416), result.get(4));
			assertEquals(new Point3D(1.571, 2.51,   3.417), result.get(13));
			assertEquals(new Point3D(1.587, 2.585,  3.467), result.get(19));
			
		} catch (IOException e) {
				fail(e.toString());
			}	
	}

}
