/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


import java.io.File;
import java.io.IOException;
import jgromacs.data.Alignment;
import jgromacs.data.IndexSet;
import jgromacs.data.IndexSetList;
import jgromacs.data.Point3D;
import jgromacs.data.Sequence;
import jgromacs.data.SequencePosition;
import jgromacs.data.Structure;
import jgromacs.data.Trajectory;
import jgromacs.db.AtomType;
import jgromacs.db.ResidueType;
import jgromacs.io.IOData;
import junit.framework.TestCase;
import org.junit.Test;

public class IODataTest extends TestCase{

	@Test
	public void testReadStructureFromGRO() {
		
		try {
		
		Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
		
		assertEquals(93, s.getNumberOfResidues()); // Number of residues read
		assertEquals(1445, s.getNumberOfAtoms()); // Number of atoms read
		assertEquals(new Point3D(1.313, 2.073, 1.827), s.getAtom(0).getCoordinates()); // Coordinates of the first atom read
		assertEquals(new Point3D(1.478, 1.755, 1.900), s.getAtom(s.getNumberOfAtoms()-1).getCoordinates()); // Coordinates of the last atom read
		assertEquals(new ResidueType("ALA"), s.getResidue(0).getResidueType()); // Type of first residue read
		assertEquals("HB3", s.getAtom(8).getName()); // Atom name of 9th atom read
		assertEquals(new AtomType("C"), s.getAtom(3).getAtomType()); // Type of 4th atom read
		
		} catch (IOException e) {
			fail(e.toString());
		}
			
	}

	@Test
	public void testReadStructureFromPDBString() {
		
		try {
		
		Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
		assertEquals(2, s.getNumberOfChains()); // Number of chains
		assertEquals(469, s.getNumberOfResidues()); // Number of residues read
		assertEquals(1811, s.getNumberOfAtoms()); // Number of atoms read
		assertEquals(new Point3D(-0.8269, 0.1925, 0.5441), s.getAtom(0).getCoordinates()); // Coordinates of the first atom read
		assertEquals(new Point3D(0.7956, 1.6854, 4.4955), s.getAtom(s.getNumberOfAtoms()-1).getCoordinates()); // Coordinates of the last atom read
		assertEquals("NE2", s.getAtom(514).getName()); // Atom name of 515th atom read
		assertEquals(new ResidueType("GLY"), s.getResidue(0).getResidueType()); // Type of first residue read
		assertEquals(new AtomType("O"), s.getAtom(9).getAtomType()); // Type of 10th atom read
		assertEquals(11.91, s.getAtom(99).getBvalue(), 0.0000001); // B-value of 100th atom read
		assertEquals(1.0, s.getAtom(99).getOccupancy(), 0.0000001); // Occupancy of 100th atom read
		
		
		Structure s2 = IOData.readStructureFromPDB("dataset//example_ensemble.pdb"); // Reading the first MODEl from example_ensemble.pdb
		assertEquals(new Point3D(2.1293, 4.4842, 2.1836), s2.getAtom(0).getCoordinates()); // Coordinates of the first atom read
		assertEquals(new Point3D(1.9275, 3.8550, 2.5984), s2.getAtom(s2.getNumberOfAtoms()-1).getCoordinates()); // Coordinates of the last atom read
		
		} catch (IOException e) {
			fail(e.toString());
		}
	}

	@Test
	public void testReadStructureFromPDBStringInt() {
		
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_ensemble.pdb", 7);  // Reading MODEL 7 from example_ensemble.pdb 
		
			assertEquals(1, s.getNumberOfChains()); // Number of chains
			assertEquals(94, s.getNumberOfResidues()); // Number of residues read
			assertEquals(1393, s.getNumberOfAtoms()); // Number of atoms read
			assertEquals(new Point3D(2.6527,4.3317,2.0140), s.getAtom(0).getCoordinates()); // Coordinates of the first atom read
			assertEquals(new Point3D(1.9789,3.7570,2.7579), s.getAtom(s.getNumberOfAtoms()-1).getCoordinates()); // Coordinates of the last atom read		
			assertEquals("HG23", s.getAtom(988).getName()); // Atom name of 989th atom read
			assertEquals(new ResidueType("LYS"), s.getResidue(0).getResidueType()); // Type of first residue read
			assertEquals(new AtomType("N"), s.getAtom(22).getAtomType()); // Type of 23th atom read
			assertEquals(0.00, s.getAtom(9).getBvalue(), 0.0000001); // B-value of 10th atom read
			assertEquals(1.0, s.getAtom(9).getOccupancy(), 0.0000001); // Occupancy of 10th atom read
			
			
			Structure s2 = IOData.readStructureFromPDB("dataset//example_single.pdb", 1);  // Reading the only model from example_single.pdb 
			
			assertEquals(2, s2.getNumberOfChains()); // Number of chains
			assertEquals(469, s2.getNumberOfResidues()); // Number of residues read
			assertEquals(1811, s2.getNumberOfAtoms()); // Number of atoms read
			assertEquals(new Point3D(-0.8269, 0.1925, 0.5441), s2.getAtom(0).getCoordinates()); // Coordinates of the first atom read
			assertEquals(new Point3D(0.7956, 1.6854, 4.4955), s2.getAtom(s2.getNumberOfAtoms()-1).getCoordinates()); // Coordinates of the last atom read
			assertEquals("NE2", s2.getAtom(514).getName()); // Atom name of 515th atom read
			assertEquals(new ResidueType("GLY"), s2.getResidue(0).getResidueType()); // Type of first residue read
			assertEquals(new AtomType("O"), s2.getAtom(9).getAtomType()); // Type of 10th atom read
			assertEquals(11.91, s2.getAtom(99).getBvalue(), 0.0000001); // B-value of 100th atom read
			assertEquals(1.0, s2.getAtom(99).getOccupancy(), 0.0000001); // Occupancy of 100th atom read
			
			Structure s3 = IOData.readStructureFromPDB("dataset//example_ensemble.pdb", 47);  // Reading a model that does not exist
			assertNull(s3);
			
			Structure s4 = IOData.readStructureFromPDB("dataset//example_single.pdb", 2);  // Reading a model that does not exist
			assertNull(s4);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testHowManyModelsInPDB() {
		
		try {
			
			int num1 = IOData.howManyModelsInPDB("dataset//example_ensemble.pdb");
			int num2 = IOData.howManyModelsInPDB("dataset//example_single.pdb");
			
			assertEquals(35, num1); // Number of models in example_ensemble.pdb
			assertEquals(1, num2); // Number of models in example_ensemble.pdb
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testReadStructuresFromPDB() {
		
		try {
			
			Structure[] ensemble = IOData.readStructuresFromPDB("dataset//example_ensemble.pdb");  // Reading structures from example_ensemble.pdb 
		
			assertEquals(35, ensemble.length); // Number of structures read
		
			// structure 7:
			assertEquals(1, ensemble[6].getNumberOfChains()); // Number of chains
			assertEquals(94, ensemble[6].getNumberOfResidues()); // Number of residues read
			assertEquals(1393, ensemble[6].getNumberOfAtoms()); // Number of atoms read
			assertEquals(new Point3D(2.6527,4.3317,2.0140), ensemble[6].getAtom(0).getCoordinates()); // Coordinates of the first atom read
			assertEquals(new Point3D(1.9789,3.7570,2.7579), ensemble[6].getAtom(ensemble[6].getNumberOfAtoms()-1).getCoordinates()); // Coordinates of the last atom read		
			assertEquals("HG23", ensemble[6].getAtom(988).getName()); // Atom name of 989th atom read
			assertEquals(new ResidueType("LYS"), ensemble[6].getResidue(0).getResidueType()); // Type of first residue read
			assertEquals(new AtomType("N"), ensemble[6].getAtom(22).getAtomType()); // Type of 23th atom read
			assertEquals(0.00, ensemble[6].getAtom(9).getBvalue(), 0.0000001); // B-value of 10th atom read
			assertEquals(1.0, ensemble[6].getAtom(9).getOccupancy(), 0.0000001); // Occupancy of 10th atom read
			
			// structure 19:
			assertEquals(new Point3D(3.2661, 4.3842, 2.4557), ensemble[18].getAtom(0).getCoordinates()); // Coordinates of the first atom read
			assertEquals(new Point3D(2.3849, 3.9426, 2.8416), ensemble[18].getAtom(ensemble[18].getNumberOfAtoms()-1).getCoordinates()); // Coordinates of the last atom read		
			
			
			Structure[] ensemble2 = IOData.readStructuresFromPDB("dataset//example_single.pdb");  // Reading the only structure from example_single.pdb 
			assertEquals(1, ensemble2.length); // Number of structures read
			
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testReadStructuresFromPDBsInDirectory() {
		
		try {
			
			Structure[] ensemble = IOData.readStructuresFromPDBsInDirectory("dataset//"); // Reading all PDB structures from directory 
		
			assertEquals(3, ensemble.length);
			
			Structure s1 = ensemble[1]; // First MODEL in example_ensemble.pdb
			assertEquals(new Point3D(2.1293, 4.4842, 2.1836), s1.getAtom(0).getCoordinates()); // Coordinates of the first atom read
			assertEquals(new Point3D(1.9275, 3.8550, 2.5984), s1.getAtom(s1.getNumberOfAtoms()-1).getCoordinates()); // Coordinates of the last atom read
			
			Structure s2 = ensemble[2]; // Structure read from example_single.pdb
			assertEquals(new Point3D(-0.8269, 0.1925, 0.5441), s2.getAtom(0).getCoordinates()); // Coordinates of the first atom read
			assertEquals(new Point3D(0.7956, 1.6854, 4.4955), s2.getAtom(s2.getNumberOfAtoms()-1).getCoordinates()); // Coordinates of the last atom read
			
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testWriteStructureToGRO() {
		try {
			
			Structure s1 = IOData.readStructureFromGRO("dataset//example_protein.gro"); // Reading from example_protein.gro
			IOData.writeStructureToGRO("temp.gro", s1); // Writing back to a temporary file
			Structure s2 = IOData.readStructureFromGRO("temp.gro"); // Reading the temporary file
			assertEquals(s1, s2); // Comparison
			File f = new File("temp.gro");
			f.delete(); // Removing temporary file
			
			} catch (IOException e) {
				fail(e.toString());
			}
	
	}

	@Test
	public void testWriteStructureToPDB() {
			
			try {
			
			Structure s1 = IOData.readStructureFromPDB("dataset//example_single.pdb"); // Reading from example_single.pdb
			IOData.writeStructureToPDB("temp.pdb", s1); // Writing back to a temporary file
			Structure s2 = IOData.readStructureFromPDB("temp.pdb"); // Reading the temporary file
			assertEquals(s1, s2); // Comparison
			File f = new File("temp.pdb");
			f.delete(); // Removing temporary file
			
			} catch (IOException e) {
				fail(e.toString());
			}
	
	
	}

	@Test
	public void testReadTrajectory() {
		
		// Note that this method depends on the GROMACS code so it will pass the test only if
		// a working copy of GROMACS is installed on your computer and gmxdump command is available from the current directory
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readTrajectory(s,"dataset//example_calpha.xtc");
	
			assertEquals(20, t.getNumberOfFrames()); // Number of frames read
			assertEquals(93, t.getNumberOfAtoms()); // Number of atoms read
			assertEquals(1000.0, t.getStartTime(),0.0001); // Start time in ps
			assertEquals(5.0, t.getTimeStep(),0.0001); // Time step in ps
		
			// Checking coordinates of different atoms in different frames:
			assertEquals(new Point3D(2.713, 3.417, 2.127),t.getFrameAsPointList(0).getPoint(21));
			assertEquals(new Point3D(3.143, 3.548, 2.379),t.getFrameAsPointList(7).getPoint(70));
			assertEquals(new Point3D(1.753, 2.726, 3.709),t.getFrameAsPointList(9).getPoint(5));
			assertEquals(new Point3D(2.208, 2.456, 1.608),t.getFrameAsPointList(19).getPoint(30));
					
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testReadDumpedTrajectory() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s,"dataset//example_calpha.dxtc");
	
			assertEquals(20, t.getNumberOfFrames()); // Number of frames read
			assertEquals(93, t.getNumberOfAtoms()); // Number of atoms read
			assertEquals(1000.0, t.getStartTime(),0.0001); // Start time in ps
			assertEquals(5.0, t.getTimeStep(),0.0001); // Time step in ps
		
			// Checking coordinates of different atoms in different frames:
			assertEquals(new Point3D(2.713, 3.417, 2.127),t.getFrameAsPointList(0).getPoint(21));
			assertEquals(new Point3D(3.143, 3.548, 2.379),t.getFrameAsPointList(7).getPoint(70));
			assertEquals(new Point3D(1.753, 2.726, 3.709),t.getFrameAsPointList(9).getPoint(5));
			assertEquals(new Point3D(2.208, 2.456, 1.608),t.getFrameAsPointList(19).getPoint(30));
					
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testReadSequenceFromFASTA() {
		
		try {
			
			Sequence seq = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta");
	
			assertEquals("2H3L:A", seq.getName()); // Name of sequence read
			assertEquals(103, seq.getSequenceLength()); // Length of sequence read
			assertEquals(new SequencePosition(1, new ResidueType("GLY")), seq.getPosition(0)); // First sequence position
			assertEquals(new SequencePosition(103, new ResidueType("SER")), seq.getPosition(102)); // Last sequence position
			assertEquals(new SequencePosition(79, new ResidueType("HIS")), seq.getPosition(78)); // 79th sequence position
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testReadAlignmentFromFASTA() {
		
		try {
			
			Alignment a = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
			
			assertEquals(5, a.getNumberOfSequences()); // Number of sequences read
			assertEquals("1M5Z_A", a.getSequence(1).getName()); // Name of 2nd sequence read
			assertEquals("3CBX_A", a.getSequence(4).getName()); // Name of 5th sequence read
			assertEquals(new SequencePosition(1, new ResidueType("GLY")), a.getSequence(2).getPosition(0)); // First sequence position of 3rd sequence
			assertEquals(new SequencePosition(8, new ResidueType("VAL")), a.getSequence(3).getPosition(10)); // 11th sequence position of 4th sequence
			assertEquals(105, a.getSequence(4).getSequenceLength()); // length of 5th sequence
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testWriteSequenceToFASTA() {
		
		try {
			
			Sequence seq1 = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta"); // Reading from example_sequence.fasta
			IOData.writeSequenceToFASTA("temp.fasta", seq1); // Writing back to a temporary file
			Sequence seq2 = IOData.readSequenceFromFASTA("temp.fasta"); // Reading the temporary file
			assertEquals(seq1, seq2); // Comparison
			File f = new File("temp.fasta");
			f.delete(); // Removing temporary file
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testWriteAlignmentToFASTA() {
		
		try {
			
			Alignment a1 = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta"); // Reading from example_alignment.fasta
			IOData.writeAlignmentToFASTA("temp.fasta", a1); // Writing back to a temporary file
			Alignment a2 = IOData.readAlignmentFromFASTA("temp.fasta"); // Reading the temporary file
			assertEquals(a1, a2); // Comparison
			File f = new File("temp.fasta");
			f.delete(); // Removing temporary file
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testReadIndexSetFromNDX() {
		
		try {
			
			IndexSet iset = IOData.readIndexSetFromNDX("dataset//example.ndx");
			
			assertEquals("set1", iset.getName()); // Name of index set read
			assertEquals(5, iset.getNumberOfIndices()); // Number of indices read
			assertEquals(20, (int)iset.getAsArrayList().get(3)); // 4th index in the set
			
			
			} catch (IOException e) {
				fail(e.toString());
			}
		
	}

	@Test
	public void testReadIndexSetListFromNDX() {
		
		try {
			
			IndexSetList isetlist = IOData.readIndexSetListFromNDX("dataset//example.ndx");
			
			assertEquals(2, isetlist.getNumberOfIndexSets()); // Number of index sets read
			assertEquals("set1", isetlist.getIndexSet(0).getName()); // Name of 1st index set read
			assertEquals("set2", isetlist.getIndexSet(1).getName()); // Name of 2nd index set read
			assertEquals(5, isetlist.getIndexSet(0).getNumberOfIndices()); // Number of indices in 1st index set
			assertEquals(10, isetlist.getIndexSet(1).getNumberOfIndices()); // Number of indices in 2nd index set
			assertEquals(20, (int)isetlist.getIndexSet(0).getAsArrayList().get(3)); // 4th index in the 1st index set
			assertEquals(73, (int)isetlist.getIndexSet(1).getAsArrayList().get(2)); // 3rd index in the 2nd index set
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testWriteIndexSetToNDX() {
		try {
			
			IndexSet is1 = IOData.readIndexSetFromNDX("dataset//example.ndx"); // Reading from example.ndx
			IOData.writeIndexSetToNDX("temp.ndx", is1); // Writing back to a temporary file
			IndexSet is2 = IOData.readIndexSetFromNDX("temp.ndx"); // Reading the temporary file
			assertEquals(is1, is2); // Comparison
			File f = new File("temp.ndx");
			f.delete(); // Removing temporary file
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testWriteIndexSetListToNDX() {
		
		try {
			
			IndexSetList isl1 = IOData.readIndexSetListFromNDX("dataset//example.ndx"); // Reading from example.ndx
			IOData.writeIndexSetListToNDX("temp.ndx", isl1); // Writing back to a temporary file
			IndexSetList isl2 = IOData.readIndexSetListFromNDX("temp.ndx"); // Reading the temporary file
			assertEquals(isl1, isl2); // Comparison
			File f = new File("temp.ndx");
			f.delete(); // Removing temporary file
			
			} catch (IOException e) {
				fail(e.toString());
			}
		
	}

	@Test
	public void testRunGromacsCommand() {
		
		// Note that this method depends on the GROMACS code so it will pass the test only if
		// a working copy of GROMACS is installed on your computer and gmxdump command is available from the current directory
		
		String[] files = {"output.ndx"};
		try {
		
			Object[] result = IOData.runGromacsCommand(
					"echo q | make_ndx -f dataset//example_single.pdb -o output.ndx", files);
			
			IndexSetList isetlist = (IndexSetList)result[1];
				
			IndexSetList control = IOData.readIndexSetListFromNDX("dataset//example_single_index.ndx");
			
			assertEquals(control, isetlist);
		
		} catch (IOException e) {
			fail(e.toString());
		}
		
		
	}

	@Test
	public void testReadFromFile() {
		try {
			
			IndexSetList isetlist = (IndexSetList)IOData.readFromFile("dataset//example.ndx");
			assertEquals(2, isetlist.getNumberOfIndexSets());
			
			Sequence seq = (Sequence)IOData.readFromFile("dataset//example_sequence.fasta");
			assertEquals(103, seq.getSequenceLength());
			
			Structure s = (Structure)IOData.readFromFile("dataset//example_protein.gro");
			assertEquals(1445, s.getNumberOfAtoms());
			
		} catch (IOException e) {
			fail(e.toString());
		}	
	}

}
