/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


import java.io.IOException;

import jgromacs.data.Alignment;
import jgromacs.data.Sequence;
import jgromacs.data.SequencePosition;
import jgromacs.db.ResidueType;
import jgromacs.io.IOData;
import junit.framework.TestCase;

import org.junit.Test;

public class SequenceTest extends TestCase{

	@Test
	public void testGetSequenceLength() {
		
		try {
			
			Sequence s = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta");
			assertEquals(103, s.getSequenceLength());
			
			Alignment a = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
			Sequence s2 = a.getSequence(3);
			assertEquals(94, s2.getSequenceLength());
			
			} catch (IOException e) {
				fail(e.toString());
			}
		
	}

	@Test
	public void testGetSequenceLengthWithGaps() {
		
		try {
	
			Alignment a = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
			
			Sequence s = a.getSequence(3);
			assertEquals(115, s.getSequenceLengthWithGaps());
						
			} catch (IOException e) {
				fail(e.toString());
			}
		
	}

	@Test
	public void testGetPositionByIndex() {
		
		try {
			
			Sequence s = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta");
			
			assertEquals(new SequencePosition(2, new ResidueType("S")), s.getPositionByIndex(2));
			assertEquals(new SequencePosition(10, new ResidueType("K")), s.getPositionByIndex(10));
			assertEquals(new SequencePosition(52, new ResidueType("P")), s.getPositionByIndex(52));
			assertEquals(new SequencePosition(70, new ResidueType("N")), s.getPositionByIndex(70));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetIndexOfPosition() {
		
		try {
			
			Sequence s = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta");
			

			assertEquals(1, s.getIndexOfPosition(0));
			assertEquals(9, s.getIndexOfPosition(8));
			assertEquals(33, s.getIndexOfPosition(32));
			assertEquals(90, s.getIndexOfPosition(89));
			
			s.reIndexPositions(100); // Reindexing positions starting from 100
		
			assertEquals(100, s.getIndexOfPosition(0));
			assertEquals(108, s.getIndexOfPosition(8));
			assertEquals(132, s.getIndexOfPosition(32));
			assertEquals(189, s.getIndexOfPosition(89));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetResidueTypeOfPositionOfIndex() {
	try {
			
			Sequence s = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta");
			
			assertEquals(new ResidueType("S"), s.getResidueTypeOfPositionOfIndex(2));
			assertEquals(new ResidueType("K"), s.getResidueTypeOfPositionOfIndex(10));
			assertEquals(new ResidueType("P"), s.getResidueTypeOfPositionOfIndex(52));
			assertEquals(new ResidueType("N"), s.getResidueTypeOfPositionOfIndex(70));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetAnnotationOfPositionOfIndex() {

		try {
			
			Sequence s = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta");
			
			assertEquals("", s.getAnnotationOfPositionOfIndex(51));
			
			SequencePosition sp = s.getPosition(50);
			sp.setAnnotation("xyz");
			s.setPosition(50, sp);
			
			assertEquals("xyz", s.getAnnotationOfPositionOfIndex(51));
		
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testAddPositionResidueTypeString() {

		try {
			
			Sequence s = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta");
			
			s.addPosition(new ResidueType("H"), "xyz"); // Adding new position (index automatically calculated)
			assertEquals(new SequencePosition(104, new ResidueType("H"),"xyz"), s.getPosition(s.getSequenceLengthWithGaps()-1));
			
			s.addGap();
			s.addGap();
			s.addGap();
			
			s.addPosition(new ResidueType("M"), "abc"); // Adding new posiion (index automatically calculated)
			assertEquals(new SequencePosition(105, new ResidueType("M"),"abc"), s.getPosition(s.getSequenceLengthWithGaps()-1));
		
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testAddPositionsFromString() {
		
		Sequence s = new Sequence();	
		
		s.addPositionsFromString("MHTYKLPPR--HNSG"); // Adding positions from String
		
		assertEquals(15, s.getSequenceLengthWithGaps());
		
		assertEquals(new ResidueType("M"), s.getPosition(0).getResidueType());
		assertEquals(new ResidueType("H"), s.getPosition(1).getResidueType());
		assertEquals(new ResidueType("T"), s.getPosition(2).getResidueType());
		assertEquals(new ResidueType("Y"), s.getPosition(3).getResidueType());
		assertEquals(new ResidueType("K"), s.getPosition(4).getResidueType());
		assertEquals(new ResidueType("L"), s.getPosition(5).getResidueType());
		assertEquals(new ResidueType("P"), s.getPosition(6).getResidueType());
		assertEquals(new ResidueType("P"), s.getPosition(7).getResidueType());
		assertEquals(new ResidueType("R"), s.getPosition(8).getResidueType());
		assertEquals(new ResidueType(), s.getPosition(9).getResidueType());
		assertEquals(new ResidueType(), s.getPosition(10).getResidueType());
		assertEquals(new ResidueType("H"), s.getPosition(11).getResidueType());
		assertEquals(new ResidueType("N"), s.getPosition(12).getResidueType());
		assertEquals(new ResidueType("S"), s.getPosition(13).getResidueType());
		assertEquals(new ResidueType("G"), s.getPosition(14).getResidueType());
	}

	@Test
	public void testAddGap() {
	
		Sequence s = new Sequence();	
		
		s.addPositionsFromString("MHTYK");  // 5 residues are already present in the sequence
		assertEquals(new ResidueType("M"), s.getPosition(0).getResidueType());
		assertEquals(new ResidueType("H"), s.getPosition(1).getResidueType());
		assertEquals(new ResidueType("T"), s.getPosition(2).getResidueType());
		assertEquals(new ResidueType("Y"), s.getPosition(3).getResidueType());
		assertEquals(new ResidueType("K"), s.getPosition(4).getResidueType());
		
		s.addGap(); // Adding new gap
		
		assertEquals(new ResidueType("M"), s.getPosition(0).getResidueType());
		assertEquals(new ResidueType("H"), s.getPosition(1).getResidueType());
		assertEquals(new ResidueType("T"), s.getPosition(2).getResidueType());
		assertEquals(new ResidueType("Y"), s.getPosition(3).getResidueType());
		assertEquals(new ResidueType("K"), s.getPosition(4).getResidueType());
		assertEquals(new ResidueType(), s.getPosition(5).getResidueType());  // Checking the added gap
	}

	@Test
	public void testInsertGap() {
		Sequence s = new Sequence();	
		
		s.addPositionsFromString("MHTYK");  // 5 residues are already present in the sequence
		assertEquals(new ResidueType("M"), s.getPosition(0).getResidueType());
		assertEquals(new ResidueType("H"), s.getPosition(1).getResidueType());
		assertEquals(new ResidueType("T"), s.getPosition(2).getResidueType());
		assertEquals(new ResidueType("Y"), s.getPosition(3).getResidueType());
		assertEquals(new ResidueType("K"), s.getPosition(4).getResidueType());
		
		s.insertGap(2); // Inserting new gap
		
		assertEquals(new ResidueType("M"), s.getPosition(0).getResidueType());
		assertEquals(new ResidueType("H"), s.getPosition(1).getResidueType());
		assertEquals(new ResidueType(), s.getPosition(2).getResidueType());  // Checking the inserted gap
		assertEquals(new ResidueType("T"), s.getPosition(3).getResidueType());
		assertEquals(new ResidueType("Y"), s.getPosition(4).getResidueType());
		assertEquals(new ResidueType("K"), s.getPosition(5).getResidueType());
		
	}

	@Test
	public void testSetPositionIntResidueTypeString() {
		try {
			
			Sequence s = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta");
			
			s.setPosition(10, new ResidueType("H"), "xyz"); // Setting new position (index automatically calculated)
			assertEquals(new SequencePosition(11, new ResidueType("H"),"xyz"), s.getPosition(10));
			
			s.insertGap(20);
			s.insertGap(20);
			s.insertGap(20);
			
			s.setPosition(23, new ResidueType("M"), "abc"); // Setting new position (index automatically calculated)
		
			assertEquals(new SequencePosition(21, new ResidueType("M"),"abc"), s.getPosition(23));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testSetPositionByIndexIntResidueTypeString() {
		try {
			
			Sequence s = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta");
			
			assertEquals(new SequencePosition(25, new ResidueType("F"),""), s.getPosition(24));
			
			s.setPositionByIndex(25, new ResidueType("H"), "xyz");
			
			assertEquals(new SequencePosition(25, new ResidueType("H"),"xyz"), s.getPosition(24));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testSetPositionByIndexIntResidueType() {
		try {
			
			Sequence s = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta");
			
			assertEquals(new SequencePosition(25, new ResidueType("F"),""), s.getPosition(24));
			
			s.setPositionByIndex(25, new ResidueType("H"));
			
			assertEquals(new SequencePosition(25, new ResidueType("H"),""), s.getPosition(24));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testRemovePositionByIndex() {
		
		Sequence s = new Sequence();	
		s.addPositionsFromString("MHTYKLP");
		
		s.removePositionByIndex(4);
		
		assertEquals(new SequencePosition(1,new ResidueType("M"),""), s.getPosition(0));
		assertEquals(new SequencePosition(2,new ResidueType("H"),""), s.getPosition(1));
		assertEquals(new SequencePosition(3,new ResidueType("T"),""), s.getPosition(2));
		assertEquals(new SequencePosition(5,new ResidueType("K"),""), s.getPosition(3));
		assertEquals(new SequencePosition(6,new ResidueType("L"),""), s.getPosition(4));
		assertEquals(new SequencePosition(7,new ResidueType("P"),""), s.getPosition(5));
		
	}

	@Test
	public void testRemoveGaps() {
		
			
			Sequence s = new Sequence("");
			s.addPositionsFromString("-HM--PKSR--GQ-YK---");
		
			assertEquals(19, s.getSequenceLengthWithGaps());
		
			s.removeGaps();
			
			Sequence control = new Sequence("");
			control.addPositionsFromString("HMPKSRGQYK");
			
			assertEquals(control, s);
			
	}

	@Test
	public void testReIndexPositionsInt() {

		Sequence s = new Sequence();	
		s.addPositionsFromString("MHTYKLP");
		
		assertEquals(1, s.getPosition(0).getIndex());
		assertEquals(2, s.getPosition(1).getIndex());
		assertEquals(3, s.getPosition(2).getIndex());
		assertEquals(4, s.getPosition(3).getIndex());
		assertEquals(5, s.getPosition(4).getIndex());
		assertEquals(6, s.getPosition(5).getIndex());
		assertEquals(7, s.getPosition(6).getIndex());
		
		s.reIndexPositions(777); // Reindexing positions starting from 777
		
		assertEquals(777, s.getPosition(0).getIndex());
		assertEquals(778, s.getPosition(1).getIndex());
		assertEquals(779, s.getPosition(2).getIndex());
		assertEquals(780, s.getPosition(3).getIndex());
		assertEquals(781, s.getPosition(4).getIndex());
		assertEquals(782, s.getPosition(5).getIndex());
		assertEquals(783, s.getPosition(6).getIndex());
		
	}

	@Test
	public void testReIndexPositions() {

		Sequence s = new Sequence();	
		s.addPositionsFromString("MHTYKLP");
			
		s.reIndexPositions(777); // Reindexing positions starting from 777
		
		assertEquals(777, s.getPosition(0).getIndex());
		assertEquals(778, s.getPosition(1).getIndex());
		assertEquals(779, s.getPosition(2).getIndex());
		assertEquals(780, s.getPosition(3).getIndex());
		assertEquals(781, s.getPosition(4).getIndex());
		assertEquals(782, s.getPosition(5).getIndex());
		assertEquals(783, s.getPosition(6).getIndex());
		
		s.reIndexPositions(); // The indices will now start from 1
		
		assertEquals(1, s.getPosition(0).getIndex());
		assertEquals(2, s.getPosition(1).getIndex());
		assertEquals(3, s.getPosition(2).getIndex());
		assertEquals(4, s.getPosition(3).getIndex());
		assertEquals(5, s.getPosition(4).getIndex());
		assertEquals(6, s.getPosition(5).getIndex());
		assertEquals(7, s.getPosition(6).getIndex());
		
	}

	@Test
	public void testInsertPositionIntResidueTypeString() {
		try {
			
			Sequence s = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta");
			
			assertEquals(new SequencePosition(10, new ResidueType("K"),""), s.getPosition(9));
			assertEquals(new SequencePosition(11, new ResidueType("Q"),""), s.getPosition(10));
			
			s.insertPosition(10, new ResidueType("H"), "xyz"); // Inserting new position (index automatically calculated)
			
			assertEquals(new SequencePosition(10, new ResidueType("K"),""), s.getPosition(9));
			assertEquals(new SequencePosition(11, new ResidueType("H"),"xyz"), s.getPosition(10));
			assertEquals(new SequencePosition(11, new ResidueType("Q"),""), s.getPosition(11)); // Re-indexing would be necessary
			
			s.insertGap(5);
			s.insertGap(5);
			s.insertGap(5);
			
			s.insertPosition(8, new ResidueType("M"), "abc"); // Inserting new position (index automatically calculated)
		
			assertEquals(new SequencePosition(6, new ResidueType("M"),"abc"), s.getPosition(8));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testIsPositionAGap() {
		
		Sequence s = new Sequence();
		s.addPositionsFromString("-HM--PKSR--GQ-YK---");
	
		assertTrue(s.isPositionAGap(0));
		assertTrue(!s.isPositionAGap(1));
		assertTrue(s.isPositionAGap(4));
		assertTrue(!s.isPositionAGap(6));
		assertTrue(s.isPositionAGap(10));
		assertTrue(!s.isPositionAGap(12));
		assertTrue(s.isPositionAGap(17));
	
	}

	@Test
	public void testGetSubSequence() {
		
		Sequence s = new Sequence();
		s.addPositionsFromString("-HM--PKSR--GQ-YK---");
		
		Sequence subs = s.getSubSequence(2, 13);
		subs.setName("");
		
		Sequence control = new Sequence("");
		control.addPositionsFromString("M--PKSR--GQ-");
		control.reIndexPositions(2);
				
		assertEquals(control, subs);
		
	}

	@Test
	public void testGetSubSequenceFrom() {
		
		Sequence s = new Sequence();
		s.addPositionsFromString("-HM--PKSR--GQ-YK---");
		
		Sequence subs = s.getSubSequenceFrom(5);
		subs.setName("");
		
		Sequence control = new Sequence("");
		control.addPositionsFromString("PKSR--GQ-YK---");
		control.reIndexPositions(3);
						
		assertEquals(control, subs);
		
	}

	@Test
	public void testGetSubSequenceTo() {
		Sequence s = new Sequence();
		s.addPositionsFromString("-HM--PKSR--GQ-YK---");
		
		Sequence subs = s.getSubSequenceTo(8);
		subs.setName("");
		
		Sequence control = new Sequence("");
		control.addPositionsFromString("-HM--PKSR");
						
		assertEquals(control, subs);
	}

	@Test
	public void testGetReverse() {
		Sequence s = new Sequence();
		s.addPositionsFromString("-HM--PKSR--GQ-YK---");
		
		Sequence revs = s.getReverse();
		revs.setName("");
		
		Sequence control = new Sequence("");
		control.addGap();
		control.addGap();
		control.addGap();
		control.addPosition(10, new ResidueType("K"));
		control.addPosition(9, new ResidueType("Y"));
		control.addGap();
		control.addPosition(8, new ResidueType("Q"));
		control.addPosition(7, new ResidueType("G"));
		control.addGap();
		control.addGap();
		control.addPosition(6, new ResidueType("R"));
		control.addPosition(5, new ResidueType("S"));
		control.addPosition(4, new ResidueType("K"));
		control.addPosition(3, new ResidueType("P"));
		control.addGap();
		control.addGap();
		control.addPosition(2, new ResidueType("M"));
		control.addPosition(1, new ResidueType("H"));
		control.addGap();
		
		assertEquals(control, revs);
	}

	@Test
	public void testConcat() {
		Sequence s1 = new Sequence();
		s1.addPositionsFromString("HMPKSR");
		
		Sequence s2 = new Sequence();
		s1.addPositionsFromString("PAHVTY");
		
		Sequence concs = s1.concat(s2);
		concs.setName("");
		
		Sequence control = new Sequence("");
		control.addPositionsFromString("HMPKSRPAHVTY");
						
		assertEquals(control, concs);
	}

	@Test
	public void testContainsSubSequence() {
		Sequence seq = new Sequence("");
		seq.addPositionsFromString("HMPK--SRPAHVTY-H");
		
		Sequence s1 = new Sequence("");
		s1.addPositionsFromString("SRPAHV");
		assertTrue(seq.containsSubSequence(s1));
		
		Sequence s2 = new Sequence("");
		s2.addPositionsFromString("TY-H");
		assertTrue(seq.containsSubSequence(s2));
		
		Sequence s3 = new Sequence("");
		s3.addPositionsFromString("PK--SRPA");
		assertTrue(seq.containsSubSequence(s3));
		
		Sequence s4 = new Sequence("");
		s4.addPositionsFromString("PKSRPA");
		assertTrue(!seq.containsSubSequence(s4));
		
		Sequence s5 = new Sequence("");
		s5.addPositionsFromString("PAHHV");
		assertTrue(!seq.containsSubSequence(s5));
	}

	@Test
	public void testToString1Letter() {
		
		Sequence seq = new Sequence("name");
		seq.addPositionsFromString("HMPK--SRPAHVTY-H");
		
		assertEquals(">name\n" +
				"HMPK--SRPAHVTY-H", seq.toString1Letter());
		
	}

	@Test
	public void testToString3Letter() {
		Sequence seq = new Sequence("name");
		seq.addPositionsFromString("HMPK--SRPAHVTY-");
		assertEquals(">name\n" +
				     "His Met Pro Lys --- --- Ser Arg Pro Ala His Val Thr Tyr ---", seq.toString3Letter());
	}

	
	@Test
	public void testCloneEquals() {
		
		Sequence x = new Sequence();
		x.addPositionsFromString("HHHHHH");
		
		Sequence cx = (Sequence)x.clone();
		
		assertTrue(x.equals(cx));
		assertTrue(x!=cx);
		
		x.addGap();
		assertTrue(!x.equals(cx));
		
	}

}
