/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


import java.io.IOException;
import java.util.ArrayList;

import jgromacs.data.Atom;
import jgromacs.data.IndexSet;
import jgromacs.data.IndexSetList;
import jgromacs.data.Point3D;
import jgromacs.data.PointList;
import jgromacs.data.Residue;
import jgromacs.data.Sequence;
import jgromacs.data.Structure;
import jgromacs.db.AtomType;
import jgromacs.db.ResidueType;
import jgromacs.io.IOData;
import junit.framework.TestCase;

import org.junit.Test;

public class StructureTest extends TestCase{

	@Test
	public void testGetNumberOfAtoms() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			assertEquals(1445, s.getNumberOfAtoms()); 		
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetNumberOfChains() {
		
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
			Structure s2 = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			assertEquals(2, s.getNumberOfChains()); 	
			assertEquals(1, s2.getNumberOfChains()); 
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetChainIDs() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			ArrayList<String> IDs =	s.getChainIDs();
			
			assertEquals("A", IDs.get(0)); 	
			assertEquals("B", IDs.get(1)); 	
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetResidueByIndexIntString() {
	
		Structure s = new Structure();
		
		s.addResidue(new Residue(1,"ARG","A", new ResidueType("R")));
		s.addResidue(new Residue(2,"HIS","A", new ResidueType("H")));
		s.addResidue(new Residue(1,"CYS","B", new ResidueType("C")));
		s.addResidue(new Residue(2,"GLY","B", new ResidueType("G")));

		assertEquals(new ResidueType("R"), s.getResidueByIndex(1, "A").getResidueType());
		assertEquals(new ResidueType("H"), s.getResidueByIndex(2, "A").getResidueType());
		assertEquals(new ResidueType("C"), s.getResidueByIndex(1, "B").getResidueType());
		assertEquals(new ResidueType("G"), s.getResidueByIndex(2, "B").getResidueType());
	}

	@Test
	public void testGetResidueByIndexInt() {
		Structure s = new Structure();
		
		s.addResidue(new Residue(1,"ARG","A", new ResidueType("R")));
		s.addResidue(new Residue(2,"HIS","A", new ResidueType("H")));
		s.addResidue(new Residue(3,"CYS","A", new ResidueType("C")));
		s.addResidue(new Residue(4,"GLY","A", new ResidueType("G")));
		s.addResidue(new Residue(5,"VAL","A", new ResidueType("V")));
		s.addResidue(new Residue(6,"LYS","A", new ResidueType("K")));

		assertEquals(new ResidueType("R"), s.getResidueByIndex(1).getResidueType());
		assertEquals(new ResidueType("C"), s.getResidueByIndex(3).getResidueType());
		assertEquals(new ResidueType("G"), s.getResidueByIndex(4).getResidueType());
		assertEquals(new ResidueType("K"), s.getResidueByIndex(6).getResidueType());
	
	}

	@Test
	public void testGetAtom() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			assertEquals("N", s.getAtom(0).getName()); 	
			assertEquals(new AtomType("N"), s.getAtom(0).getAtomType()); 	
			assertEquals(1, s.getAtom(0).getIndex()); 	
			
			assertEquals("CB", s.getAtom(55).getName()); 	
			assertEquals(new AtomType("C"), s.getAtom(55).getAtomType()); 	
			assertEquals(56, s.getAtom(55).getIndex()); 	
			
			assertEquals("CD2", s.getAtom(650).getName()); 	
			assertEquals(new AtomType("C"), s.getAtom(650).getAtomType()); 	
			assertEquals(651, s.getAtom(650).getIndex()); 	
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetAtomByIndex() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			assertEquals("N", s.getAtomByIndex(1).getName()); 	
			assertEquals(new AtomType("N"), s.getAtomByIndex(1).getAtomType()); 	
			assertEquals(1, s.getAtomByIndex(1).getIndex()); 	
			
			assertEquals("CB", s.getAtomByIndex(56).getName()); 	
			assertEquals(new AtomType("C"), s.getAtomByIndex(56).getAtomType()); 	
			assertEquals(56, s.getAtomByIndex(56).getIndex()); 	
			
			assertEquals("CD2", s.getAtomByIndex(651).getName()); 	
			assertEquals(new AtomType("C"), s.getAtomByIndex(651).getAtomType()); 	
			assertEquals(651, s.getAtomByIndex(651).getIndex()); 	
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetAtomInResidueOfIndexIntStringInt() {
		
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			Atom a = s.getAtomInResidueOfIndex(1330, "A", 4);
			
			assertEquals(166, a.getIndex());
			assertEquals("CB", a.getName());
			assertEquals(new AtomType("C"), a.getAtomType());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetAtomInResidueOfIndexIntInt() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			Atom a = s.getAtomInResidueOfIndex(1330, 4);
			
			assertEquals(166, a.getIndex());
			assertEquals("CB", a.getName());
			assertEquals(new AtomType("C"), a.getAtomType());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testRemoveAtomInt() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			Atom a1 = s.getAtom(111);
			Atom a2 = s.getAtom(113);
			
			int numofatoms = s.getNumberOfAtoms();
			
			s.removeAtom(112);
			
			assertEquals(numofatoms-1, s.getNumberOfAtoms());
			
			assertEquals(a1, s.getAtom(111));
			assertEquals(a2, s.getAtom(112));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testRemoveAtomByIndex() {
		
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
				
			int numofatoms = s.getNumberOfAtoms();
			
			assertNotNull(s.getAtomByIndex(99));
			
			s.removeAtomByIndex(99);
			
			assertEquals(numofatoms-1, s.getNumberOfAtoms());

			assertNull(s.getAtomByIndex(99));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testRemoveAtomAtom() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
				
			int numofatoms = s.getNumberOfAtoms();
			
			Atom atom = s.getAtomByIndex(99);
			
			assertNotNull(atom);
			
			s.removeAtom(atom);
			
			assertEquals(numofatoms-1, s.getNumberOfAtoms());

			assertNull(s.getAtomByIndex(99));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testAddAtomToResidue() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");

			Residue r = s.getResidue(10);
			assertEquals(9, r.getNumberOfAtoms());
			
			Atom newatom = new Atom();
			newatom.setName("xyz");
			newatom.setCoordinates(new Point3D(1,1,1));
			newatom.setIndex(999);
			newatom.setAtomType(new AtomType("K"));
			
			s.addAtomToResidue(10, newatom);
			
			r = s.getResidue(10);
			assertEquals(10, r.getNumberOfAtoms());
			assertEquals(newatom, r.getAtom(r.getNumberOfAtoms()-1));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testAddAtomToResidueOfIndex() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");

			Residue r = s.getResidueByIndex(1320);
			assertEquals(9, r.getNumberOfAtoms());
			
			Atom newatom = new Atom();
			newatom.setName("xyz");
			newatom.setCoordinates(new Point3D(1,1,1));
			newatom.setIndex(999);
			newatom.setAtomType(new AtomType("K"));
			
			s.addAtomToResidueOfIndex(1320, newatom);
			
			r = s.getResidueByIndex(1320);
			assertEquals(10, r.getNumberOfAtoms());
			assertEquals(newatom, r.getAtom(r.getNumberOfAtoms()-1));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testSetAtomInResidue() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");

			Residue r = s.getResidue(10);
			
			assertEquals(78, r.getAtom(4).getIndex());
			
			Atom newatom = new Atom();
			newatom.setName("xyz");
			newatom.setCoordinates(new Point3D(1,1,1));
			newatom.setIndex(999);
			newatom.setAtomType(new AtomType("K"));
			
			s.setAtomInResidue(10, 4, newatom);
			
			r = s.getResidue(10);
			assertEquals(9, r.getNumberOfAtoms());
			assertEquals(newatom, r.getAtom(4));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testSetAtomInResidueOfIndex() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");

			Residue r = s.getResidueByIndex(1320);
			assertEquals(78, r.getAtom(4).getIndex());
			
			Atom newatom = new Atom();
			newatom.setName("xyz");
			newatom.setCoordinates(new Point3D(1,1,1));
			newatom.setIndex(999);
			newatom.setAtomType(new AtomType("K"));
			
			s.setAtomInResidueOfIndex(1320, 4, newatom);
			
			r = s.getResidueByIndex(1320);
			assertEquals(9, r.getNumberOfAtoms());
			assertEquals(newatom, r.getAtom(4));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetSystemIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_single.gro");

			IndexSet result = s.getSystemIndexSet();
			
			// Control index set is output of the make_ndx Gromacs command:
			IndexSetList isetlist = IOData.readIndexSetListFromNDX("dataset//example_single_index.ndx");
			IndexSet control = isetlist.getIndexSet("System");
			
			assertEquals(control, result);
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetProteinIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_single.gro");

			IndexSet result = s.getProteinIndexSet();

			// Control index set is output of the make_ndx Gromacs command:
			IndexSetList isetlist = IOData.readIndexSetListFromNDX("dataset//example_single_index.ndx");
			IndexSet control = isetlist.getIndexSet("Protein");
			
			assertEquals(control, result);
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetHeavyProteinIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_single.gro");

			IndexSet result = s.getHeavyProteinIndexSet();
			
			// Control index set is output of the make_ndx Gromacs command:
			IndexSetList isetlist = IOData.readIndexSetListFromNDX("dataset//example_single_index.ndx");
			IndexSet control = isetlist.getIndexSet("Protein-H");
			
			assertEquals(control, result);
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetAlphaCarbonIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_single.gro");

			IndexSet result = s.getAlphaCarbonIndexSet();
			
			// Control index set is output of the make_ndx Gromacs command:
			IndexSetList isetlist = IOData.readIndexSetListFromNDX("dataset//example_single_index.ndx");
			IndexSet control = isetlist.getIndexSet("C-alpha");
			
			assertEquals(control, result);
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetBackboneIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_single.gro");

			IndexSet result = s.getBackboneIndexSet();
		
			// Control index set is output of the make_ndx Gromacs command:
			IndexSetList isetlist = IOData.readIndexSetListFromNDX("dataset//example_single_index.ndx");
			IndexSet control = isetlist.getIndexSet("Backbone");
			
			assertEquals(control, result);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetMainChainIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_single.gro");

			IndexSet result = s.getMainChainIndexSet();
			
			// Control index set is output of the make_ndx Gromacs command:
			IndexSetList isetlist = IOData.readIndexSetListFromNDX("dataset//example_single_index.ndx");
			IndexSet control = isetlist.getIndexSet("MainChain");
			
			assertEquals(control, result);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetMainChainPlusCbIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_single.gro");

			IndexSet result = s.getMainChainPlusCbIndexSet();
		
			// Control index set is output of the make_ndx Gromacs command:
			IndexSetList isetlist = IOData.readIndexSetListFromNDX("dataset//example_single_index.ndx");
			IndexSet control = isetlist.getIndexSet("MainChain+Cb");
			
			assertEquals(control, result);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetMainChainPlusHIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_single.gro");

			IndexSet result = s.getMainChainPlusHIndexSet();
			
			// Control index set is output of the make_ndx Gromacs command:
			IndexSetList isetlist = IOData.readIndexSetListFromNDX("dataset//example_single_index.ndx");
			IndexSet control = isetlist.getIndexSet("MainChain+H");
			
			assertEquals(control, result);
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetSideChainIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_single.gro");

			IndexSet result = s.getSideChainIndexSet();
			
			// Control index set is output of the make_ndx Gromacs command:
			IndexSetList isetlist = IOData.readIndexSetListFromNDX("dataset//example_single_index.ndx");
			IndexSet control = isetlist.getIndexSet("SideChain");
			
			assertEquals(control, result);
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetSideChainMinusHIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_single.gro");

			IndexSet result = s.getSideChainMinusHIndexSet();
			
			// Control index set is output of the make_ndx Gromacs command:
			IndexSetList isetlist = IOData.readIndexSetListFromNDX("dataset//example_single_index.ndx");
			IndexSet control = isetlist.getIndexSet("SideChain-H");
			
			assertEquals(control, result);
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetNonProteinIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_single.gro");

			IndexSet result = s.getNonProteinIndexSet();
			
			// Control index set is output of the make_ndx Gromacs command:
			IndexSetList isetlist = IOData.readIndexSetListFromNDX("dataset//example_single_index.ndx");
			IndexSet control = isetlist.getIndexSet("non-Protein");
			
			assertEquals(control, result);
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetWaterIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_single.gro");

			IndexSet result = s.getWaterIndexSet();

			// Control index set is output of the make_ndx Gromacs command:
			IndexSetList isetlist = IOData.readIndexSetListFromNDX("dataset//example_single_index.ndx");
			IndexSet control = isetlist.getIndexSet("Water");
			
			assertEquals(control, result);
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetIndexSetOfChainID() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");

			IndexSet isetA = s.getIndexSetOfChainID("A");
			IndexSet isetB = s.getIndexSetOfChainID("B");
			
			assertEquals(904, isetA.getNumberOfIndices());
			assertEquals(907, isetB.getNumberOfIndices());
			
			for (int i = 1; i < 790; i++)
			assertTrue(isetA.isIndexIn(i));
			for (int i = 1546; i < 1661; i++)
			assertTrue(isetA.isIndexIn(i));
			
			for (int i = 791; i < 1545; i++)
			assertTrue(isetB.isIndexIn(i));
			for (int i = 1661; i < 1814; i++)
			assertTrue(isetB.isIndexIn(i));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testSetAtomCoordinates() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");

			assertEquals(new Point3D(0.7481, -0.2406, 2.4989), s.getAtom(200).getCoordinates());
			
			s.setAtomCoordinates(200, new Point3D(1,1,1));
			
			assertEquals(new Point3D(1,1,1), s.getAtom(200).getCoordinates());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testSetAtomOfIndexCoordinates() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");

			assertEquals(new Point3D(0.7481, -0.2406, 2.4989), s.getAtomByIndex(201).getCoordinates());
			
			s.setAtomOfIndexCoordinates(201, new Point3D(1,1,1));
			
			assertEquals(new Point3D(1,1,1), s.getAtomByIndex(201).getCoordinates());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testSetAllAtomCoordinates() {
		Structure s = new Structure();
		
		Residue r1 = new Residue();
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		Residue r2 = new Residue();
		r2.addAtom(new Atom());
		r2.addAtom(new Atom());
	
		s.addResidue(r1);
		s.addResidue(r2);
		
		PointList points = new PointList();
		points.addPoint(new Point3D(1,1,1));
		points.addPoint(new Point3D(2,2,2));
		points.addPoint(new Point3D(3,3,3));
		points.addPoint(new Point3D(4,4,4));
		
		s.setAllAtomCoordinates(points);
		
		assertEquals(new Point3D(1,1,1), s.getAtom(0).getCoordinates());
		assertEquals(new Point3D(2,2,2), s.getAtom(1).getCoordinates());
		assertEquals(new Point3D(3,3,3), s.getAtom(2).getCoordinates());
		assertEquals(new Point3D(4,4,4), s.getAtom(3).getCoordinates());
	}

	@Test
	public void testGetAllAtomCoordinates() {
		Structure s = new Structure();
		
		Residue r1 = new Residue();
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		Residue r2 = new Residue();
		r2.addAtom(new Atom());
		r2.addAtom(new Atom());
	
		s.addResidue(r1);
		s.addResidue(r2);
		
		PointList points = new PointList();
		points.addPoint(new Point3D(1,1,1));
		points.addPoint(new Point3D(2,2,2));
		points.addPoint(new Point3D(3,3,3));
		points.addPoint(new Point3D(4,4,4));
		
		s.setAllAtomCoordinates(points);
		
		assertEquals(new Point3D(1,1,1), s.getAtom(0).getCoordinates());
		assertEquals(new Point3D(2,2,2), s.getAtom(1).getCoordinates());
		assertEquals(new Point3D(3,3,3), s.getAtom(2).getCoordinates());
		assertEquals(new Point3D(4,4,4), s.getAtom(3).getCoordinates());
		
		PointList points2 = s.getAllAtomCoordinates();
		
		assertEquals(points2, points);
	}

	@Test
	public void testGetSequence() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
			Structure[] chains = s.getChains();
			
			
			Sequence control = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta");
			control.setName("");
			
			Sequence seq = chains[0].getSequence();
			seq.setName("");
			seq.reIndexPositions();
				
			assertEquals(control, seq);
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testGetChains() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");

			Structure[] chains = s.getChains();
			
			assertEquals(2, chains.length);	
			assertEquals(904, chains[0].getNumberOfAtoms());
			assertEquals(907, chains[1].getNumberOfAtoms());
			
			for (int i = 0; i < chains[0].getNumberOfResidues(); i++) 
			assertEquals("A", chains[0].getResidue(i).getChainID());
			
			for (int i = 0; i < chains[1].getNumberOfResidues(); i++) 
			assertEquals("B", chains[1].getResidue(i).getChainID());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetSubStructure() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
		
			IndexSet iset = new IndexSet();
			iset.addIndex(7);
			iset.addIndex(15);
			iset.addIndex(27);
			Structure subs = s.getSubStructure(iset);
			
			assertEquals(3, subs.getNumberOfAtoms());
			
			assertEquals(s.getAtomByIndex(7), subs.getAtom(0));
			assertEquals(s.getAtomByIndex(15), subs.getAtom(1));
			assertEquals(s.getAtomByIndex(27), subs.getAtom(2));
			
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testConvertIndicesToArrayListIndices() {
		Structure s = new Structure();
		
		// Creating first residue
		Residue r1 = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(77);
		Atom a2 = new Atom();
		a2.setIndex(78);
		Atom a3 = new Atom();
		a3.setIndex(79);		
		r1.addAtom(a1);
		r1.addAtom(a2);
		r1.addAtom(a3);
	
		// Creating second residue
		Residue r2 = new Residue();
		Atom a4 = new Atom();
		a4.setIndex(80);
		Atom a5 = new Atom();
		a5.setIndex(81);
		Atom a6 = new Atom();
		a6.setIndex(82);		
		r2.addAtom(a4);
		r2.addAtom(a5);
		r2.addAtom(a6);
	
		//The structure will contain 2 residues (6 atoms)
		s.addResidue(r1);
		s.addResidue(r2);
		
		IndexSet iset = new IndexSet();
		iset.addIndex(78);
		iset.addIndex(80);
		iset.addIndex(81);
		
		ArrayList<Integer> list = s.convertIndicesToArrayListIndices(iset);
	
		assertEquals(1, (int)list.get(0));
		assertEquals(3, (int)list.get(1));
		assertEquals(4, (int)list.get(2));
	}

	@Test
	public void testMapAtomIndicesToResidueIndices() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
		
			IndexSet iset = new IndexSet();
			iset.addIndex(108); // 6
			iset.addIndex(340); // 22
			iset.addIndex(943); // 63
			iset.addIndex(1400); // 91
			iset.addIndex(1500); // -1
			
			ArrayList<Integer> result = s.mapAtomIndicesToResidueIndices(iset);
			
			assertEquals(6, (int)result.get(0));
			assertEquals(22, (int)result.get(1));
			assertEquals(63, (int)result.get(2));
			assertEquals(91, (int)result.get(3));
			assertEquals(-1, (int)result.get(4));
			
			} catch (IOException e) {
				fail(e.toString());
			}

	}

	@Test
	public void testMapResidueIndicesToAtomIndices() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
		
			ArrayList<Integer> resindices = new ArrayList<Integer>();
			
			resindices.add(3);
			resindices.add(6);
			
			IndexSet iset = s.mapResidueIndicesToAtomIndices(resindices);
			
			assertEquals(41, iset.getNumberOfIndices());
			
			for (int i = 34; i < 51; i++)
			assertTrue(iset.isIndexIn(i));
			
			for (int i = 85; i < 109; i++)
			assertTrue(iset.isIndexIn(i));
				
			
			} catch (IOException e) {
				fail(e.toString());
			}


	}

	@Test
	public void testToStringAsGRO() {
	
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
		
			IndexSet iset = new IndexSet();
			iset.addIndex(7);
			iset.addIndex(15);
			iset.addIndex(27);
			s = s.getSubStructure(iset);
			s.setName("Anything");
			
			StringBuffer control = new StringBuffer();
			control.append("Anything\n");
			control.append("    3\n");
			control.append("    1ALA    HB1    7   1.117   1.906   1.923\n");
			control.append("    2LYSH    HA   15   1.497   2.242   2.258\n");
			control.append("    2LYSH   HE2   27   1.380   2.656   1.936\n");
			control.append("   0.00000   0.00000   0.00000\n");
			
			assertEquals(control.toString(), s.toStringAsGRO());
			
			} catch (IOException e) {
				fail(e.toString());
			}
		
	
	}

	@Test
	public void testToStringAsPDB() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			IndexSet iset = new IndexSet();
			iset.addIndex(7);
			iset.addIndex(15);
			iset.addIndex(27);
			s = s.getSubStructure(iset);
			s.setName("Anything");
			
			StringBuffer control = new StringBuffer();
			control.append("TITLE     Anything\n");
			control.append("ATOM      7  C   SER A1311      -5.372  -2.332   6.977  1.00 14.64           C\n");
			control.append("ATOM     15  CB  HIS A1312      -7.544  -5.078   6.850  1.00 13.25           C\n");
			control.append("ATOM     27  SD  MET A1313      -8.066  -2.319  -0.472  1.00 30.25           S\n");
			
			assertEquals(control.toString(), s.toStringAsPDB());
			
			} catch (IOException e) {
				fail(e.toString());
			}
		
	}
	
	@Test
	public void testCloneEquals() {
		
		try {
			
			Structure x = IOData.readStructureFromGRO("dataset//example_protein.gro");
			 		
			Structure cx = (Structure)x.clone();
			
			assertTrue(x.equals(cx));
			assertTrue(x!=cx);
			
			x.removeAtom(0);
			assertTrue(!x.equals(cx));
			
			} catch (IOException e) {
				fail(e.toString());
			}
		
				
	}

}
